﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace RobotBehaviourEditor.MotionEditor.Configuration
{
    public partial class ConfigForm : Form
    {
        public ConfigForm()
        {
            InitializeComponent();
            

            ProgramConfiguration config = ProgramConfiguration.Load(Program.ConfigFileName);
            agentFileText.Text= config.AgentFileName;
            serverFileText.Text = config.ServerFileName;
            libraryPathText.Text = config.LibraryPath;
            models3dPathText.Text = config.Models3dPath;
            agentServerIpText.Text = config.AgentServerIp;
            agentArgumentsText.Text = config.AgentArguments;            
            
            showCenterOfMassCheck.Checked = config.VisualisationCenterOfMass;

            noConsoleCheck.Checked = config.AgentNoConsole;
            noDebugCheck.Checked = config.AgentNoDebug;

            LoadModelFileNames();

            string model = (config.Visualisation3dModel ?? "").ToLower();
            editorModelRadio.Checked = (model == "editor" || string.IsNullOrEmpty(model));
            naoModelRadio.Checked = (model == "nao");
            customModelRadio.Checked = !editorModelRadio.Checked && !naoModelRadio.Checked;
            if (customModelRadio.Checked)
                customModelCombo.Text = model;            
        }

        private void LoadModelFileNames()
        {
            string[] files = Directory.GetFiles(models3dPathText.Text, "*.r3d");
            foreach (var f in files)
            {
                customModelCombo.Items.Add(Path.GetFileName(f));
            }
        }


        private void okButton_Click(object sender, EventArgs e)
        {
            ProgramConfiguration config = new ProgramConfiguration();
            config.ServerFileName = serverFileText.Text;
            config.AgentFileName = agentFileText.Text;
            config.LibraryPath = libraryPathText.Text;
            config.Models3dPath = models3dPathText.Text;
            config.AgentServerIp = agentServerIpText.Text;
            config.AgentArguments = agentArgumentsText.Text;            

            if (editorModelRadio.Checked) config.Visualisation3dModel = "editor";
            else if (naoModelRadio.Checked) config.Visualisation3dModel = "nao";
            else
            {
                config.Visualisation3dModel = customModelCombo.Text;
            }
            config.VisualisationCenterOfMass = showCenterOfMassCheck.Checked;

            config.AgentNoConsole = noConsoleCheck.Checked;
            config.AgentNoDebug = noDebugCheck.Checked;
            
            config.Save(Program.ConfigFileName);
            Program.LoadConfiguration();
        }

        private void ConfigForm_Load(object sender, EventArgs e)
        {

        }

        private void customModelCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            customModelRadio.Checked = true;
        }        
                
    }
}
