﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace RobotBehaviourEditor.MotionEditor.Configuration
{
    public class ProgramConfiguration
    {        
        public string AgentFileName { get; set; }
        public string ServerFileName { get; set; }        
        public string TempFileName { get; set; }
        public string LibraryPath { get; set; }
        public string Models3dPath { get; set; }        

        public string AgentServerIp { get; set; }
        public string AgentArguments { get; set; }
        public bool AgentNoDebug { get; set; }
        public bool AgentNoConsole { get; set; }

        public string Visualisation3dModel { get; set; }
        public bool VisualisationCenterOfMass { get; set; }

        public ProgramConfiguration()
        {
            Models3dPath = @"RobotModels\";
            LibraryPath = @"Library\";
            AgentFileName = @".\Robocup3D.exe";
            ServerFileName = @"..\Server\simspark.exe";
        }

        private enum LoadingState
        {
            Start = 0,
            Configuration = 1,
        }

        public static ProgramConfiguration Load(string configFileName)
        {
            ProgramConfiguration retval = new ProgramConfiguration();
            if (!System.IO.File.Exists(configFileName)) return retval;

            StreamReader sr = new StreamReader(configFileName, Encoding.UTF8, true);

            LoadingState state = LoadingState.Start;
            string line = "";
            while ((line = sr.ReadLine()) != null)
            {
                line = line.Trim();

                switch (line)
                {
                    case "[CONFIG]":
                        state = LoadingState.Configuration;
                        break;
                }

                int valueStart = line.IndexOf('=') + 1;
                string value = line.Substring(valueStart);
                switch (state)
                {
                    case LoadingState.Configuration:
                        if (line.StartsWith("SERVER_FILE="))
                            retval.ServerFileName = value;
                        else if (line.StartsWith("AGENT_FILE="))
                            retval.AgentFileName = value;
                        else if (line.StartsWith("TEMP_FILE="))
                            retval.TempFileName = value;
                        else if (line.StartsWith("LIBRARY_PATH="))
                            retval.LibraryPath = value;
                        else if (line.StartsWith("AGENT_ARGS="))
                            retval.AgentArguments = value;
                        else if (line.StartsWith("AGENT_SERVER_IP="))
                            retval.AgentServerIp = value;
                        else if (line.StartsWith("AGENT_NO_CONSOLE="))
                            retval.AgentNoConsole = bool.Parse(value);
                        else if (line.StartsWith("AGENT_NO_DEBUG="))
                            retval.AgentNoDebug = bool.Parse(value);
                        else if (line.StartsWith("VISUAL_CENTER_OF_MASS="))
                            retval.VisualisationCenterOfMass = bool.Parse(value);
                        else if (line.StartsWith("VISUAL_MODEL="))
                            retval.Visualisation3dModel = value;

                        break;
                    default:
                        break;
                }

            }

            sr.Close();
            return retval;
        }

        public void Save(string configFileName)
        {
            StreamWriter sw = new StreamWriter(configFileName, false, Encoding.UTF8);
            sw.WriteLine("[CONFIG]");
            sw.WriteLine("SERVER_FILE=" + ServerFileName);
            sw.WriteLine("AGENT_FILE=" + AgentFileName);
            sw.WriteLine("TEMP_FILE=" + TempFileName);
            sw.WriteLine("LIBRARY_PATH=" + LibraryPath);
            sw.WriteLine("AGENT_ARGS=" + AgentArguments);
            sw.WriteLine("AGENT_SERVER_IP=" + AgentServerIp);
            sw.WriteLine("AGENT_NO_DEBUG=" + AgentNoDebug);
            sw.WriteLine("AGENT_NO_CONSOLE=" + AgentNoConsole);
            sw.WriteLine("VISUAL_CENTER_OF_MASS=" + VisualisationCenterOfMass.ToString());
            sw.WriteLine("VISUAL_MODEL=" + Visualisation3dModel);            

            sw.Close();
        }

    }
}
