﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace RobotBehaviourEditor.Formats
{
    /**
     *  A class representation of a &lt;phase&gt; tag in Dream Team's XML
     * */
    class PhaseNode
    {
        private List<EffectorNode> moves = new List<EffectorNode>();

        public string name { get; set; }

        public string nextName { get; set; }

        public bool isFinal { get; set; }

        public int speedConstant { get; set; }
        
        public PhaseNode()
        {
            this.isFinal = false;
        }
        
        internal void addMove(EffectorNode node)
        {
            moves.Add(node);
        }

        internal List<EffectorNode> getMoves()
        {
            return new List<EffectorNode>(moves);
        }

        internal static PhaseNode parse(XmlReader reader)
        {
            PhaseNode node = new PhaseNode();
            while (reader.MoveToNextAttribute())
            {
                if ("name".Equals(reader.Name))
                    node.name = reader.Value;
                if ("next".Equals(reader.Name))
                    node.nextName = reader.Value;
                if ("is_final".Equals(reader.Name))
                    node.isFinal = "true".Equals(reader.Value);
            }
            return node;
        }

        bool equals(Object o)
        {
            if (o == null)
                return false;
            return ((PhaseNode)o).name.Equals(this.name);
        }

        string ToString()
        {
            return "PhaseNode:" +
               " name: " + name +
               " nextName: " + nextName +
               " isFinal: " + isFinal +
               " speed: " + speedConstant;
        }       
    }
}
