﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RobotBehaviourEditor.Model;
using RobotBehaviourEditor.MotionEditor;
using System.Xml;
using System.Windows.Forms;


namespace RobotBehaviourEditor.Formats
{
    class XmlExporter
    {
        private string filePath;
        private List<JointMotion> motions4exp;

        public XmlExporter(string filePath, List<JointMotion> motions)
        {

            this.filePath = filePath;
            this.motions4exp = motions;

        }

        public void exportToXMLFile()
        {
            XmlTextWriter textWriter = new XmlTextWriter(filePath, System.Text.Encoding.UTF8);
            textWriter.Formatting = Formatting.Indented;

            // Opens the document

            textWriter.WriteStartDocument();

            // Write first element
            textWriter.WriteStartElement("robot");
            textWriter.WriteStartElement("phases");

            int p = 0;
            do
            {

                // sort pola obsahujuceho vsetky JointMtiony podla zaciatocneho casu
                motions4exp.Sort(delegate(JointMotion jointM1, JointMotion jointM2)
                {
                    return jointM1.StartTime.CompareTo(jointM2.StartTime);
                });
                System.Console.Out.WriteLine("Vsetky Fazy:");
                for (int i = 0; i < motions4exp.Count; i++)
                {

                    System.Console.Out.Write(motions4exp.ElementAt(i).StartTime);
                    System.Console.Out.Write(" - ");
                    System.Console.Out.WriteLine(motions4exp.ElementAt(i).EndTime);


                }
                System.Console.Out.Write("Koniec\n");
                if (motions4exp.Count == 0) break;
                int minEndTime = motions4exp.ElementAt(0).EndTime;
                int minStartTime = motions4exp.ElementAt(0).StartTime;

                for (int i = 0; i < motions4exp.Count; i++)         //cyklus sluziaci na najdenie zaciatku a konca fazy (minStartTime a minEndTime)
                {
                    if (motions4exp.ElementAt(i).StartTime < minEndTime && motions4exp.ElementAt(i).StartTime > minStartTime)
                    {
                        minEndTime = motions4exp.ElementAt(i).StartTime;
                    }
                    if (motions4exp.ElementAt(i).EndTime < minEndTime)
                    {
                        minEndTime = motions4exp.ElementAt(i).EndTime;
                    }


                }
                // Write next element

                textWriter.WriteStartElement("phase");
                textWriter.WriteAttributeString("name", "Phase" + p);
                p++;
                if ((motions4exp.ElementAt(0).EndTime != motions4exp.ElementAt(motions4exp.Count - 1).EndTime))
                {
                    textWriter.WriteAttributeString("next", "Phase" + p);
                }
                else
                {
                    textWriter.WriteAttributeString("next", "Phase" + p);
                    textWriter.WriteAttributeString("isFinal", "true");
                }

                textWriter.WriteStartElement("efectors");


                int k = 0;
                do
                {
                    int done = 0;
                    if (motions4exp.ElementAt(k).StartTime == minStartTime)
                    {


                        if (done == 0 && motions4exp.ElementAt(k).EndTime == minEndTime)
                        {
                            // identifikovana faza (minimalna zaciatocna poloha a minimalna koncova poloha, treba fazu zapisat do XML)

                            // TODO zapis konkretnej fazy do XML
                            textWriter.WriteStartElement("effector");
                            textWriter.WriteAttributeString("name", ParseJointName(Convert.ToString(motions4exp.ElementAt(k).Joint)));



                            textWriter.WriteStartElement("start");
                            textWriter.WriteString(Convert.ToString((double)motions4exp.ElementAt(k).StartPosition));
                            textWriter.WriteEndElement();


                            textWriter.WriteStartElement("end");
                            textWriter.WriteString(Convert.ToString((double)motions4exp.ElementAt(k).EndPosition));
                            textWriter.WriteEndElement();

                            textWriter.WriteEndElement();
                            // TODO zapis konkretnej fazy do XML

                            System.Console.Out.Write("Faza typu rovna sa");
                            System.Console.Out.Write(motions4exp.ElementAt(k).StartTime);
                            System.Console.Out.Write("-");
                            System.Console.Out.Write(motions4exp.ElementAt(k).EndTime);
                            System.Console.Out.Write("\n");
                            motions4exp.Remove(motions4exp.ElementAt(k));

                            done = 1;
                        }
                        if (done == 0 && motions4exp.ElementAt(k).EndTime < minEndTime)
                        {
                            // identifikovana faza (minimalna zaciatocna poloha a minimalna koncova poloha, treba fazu zapisat do XML)

                            // TODO zapis konkretnej fazy do XML
                            textWriter.WriteStartElement("effector");
                            textWriter.WriteAttributeString("name", ParseJointName(Convert.ToString(motions4exp.ElementAt(k).Joint)));



                            textWriter.WriteStartElement("start");
                            textWriter.WriteString(Convert.ToString((double)motions4exp.ElementAt(k).StartPosition));
                            textWriter.WriteEndElement();


                            textWriter.WriteStartElement("end");
                            textWriter.WriteString(Convert.ToString((double)motions4exp.ElementAt(k).EndPosition));
                            textWriter.WriteEndElement();

                            textWriter.WriteEndElement();
                            // TODO zapis konkretnej fazy do XML

                            System.Console.Out.Write("Faza typu mensia");
                            System.Console.Out.Write(motions4exp.ElementAt(k).StartTime);
                            System.Console.Out.Write("-");
                            System.Console.Out.Write(motions4exp.ElementAt(k).EndTime);
                            System.Console.Out.Write("\n");
                            motions4exp.Remove(motions4exp.ElementAt(k));

                            done = 1;

                        }
                        if (done == 0 && motions4exp.ElementAt(k).EndTime > minEndTime)
                        {
                            // identifikovana faza (minimalna zaciatocna poloha a minimalna koncova poloha, treba fazu zapisat do XML)



                            System.Console.Out.Write("Faza typu vacsia (pred rozkladom)");
                            System.Console.Out.Write(motions4exp.ElementAt(k).StartTime);
                            System.Console.Out.Write("-");
                            System.Console.Out.Write(motions4exp.ElementAt(k).EndTime);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("StartPos " + motions4exp.ElementAt(k).StartPosition);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("EndPos" + motions4exp.ElementAt(k).EndPosition);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("Dur " + motions4exp.ElementAt(k).Duration);
                            System.Console.Out.Write("\n");

                            //hodnoty na zapisanie (sucasne) - tie sa zapisu do XML - 1. cast rozseknutia
                            int xmlDuration = minEndTime - motions4exp.ElementAt(k).StartTime;
                            int xmlEndTime = minEndTime;
                            double xmlEndPosition = Math.Round((double)xmlDuration / (double)motions4exp.ElementAt(k).Duration * (motions4exp.ElementAt(k).EndPosition - motions4exp.ElementAt(k).StartPosition)) + motions4exp.ElementAt(k).StartPosition;

                            System.Console.Out.Write("jedna " + (xmlDuration / motions4exp.ElementAt(k).Duration));
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("dva" + (motions4exp.ElementAt(k).EndPosition - motions4exp.ElementAt(k).StartPosition));
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("StartPosition " + motions4exp.ElementAt(k).StartPosition);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("EndPosition" + motions4exp.ElementAt(k).EndPosition);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("Duration " + motions4exp.ElementAt(k).Duration);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("xmlendPosition" + xmlEndPosition);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("xmlendDuration " + xmlDuration);
                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("Faza typu vacsia (prva cast)");
                            System.Console.Out.Write(motions4exp.ElementAt(k).StartTime);
                            System.Console.Out.Write("-");
                            System.Console.Out.WriteLine(xmlEndTime);
                            System.Console.Out.Write("Duration" + xmlDuration);

                            System.Console.Out.Write("\n");
                            System.Console.Out.Write("**" + motions4exp.ElementAt(k).StartTime);
                            System.Console.Out.Write("-");
                            System.Console.Out.Write("**" + motions4exp.ElementAt(k).EndTime);

                            // TODO zapis konkretnej fazy do XML
                            textWriter.WriteStartElement("effector");
                            textWriter.WriteAttributeString("name", ParseJointName(Convert.ToString(motions4exp.ElementAt(k).Joint)));

                            textWriter.WriteStartElement("start");
                            textWriter.WriteString(Convert.ToString((double)motions4exp.ElementAt(k).StartPosition));
                            textWriter.WriteEndElement();

                            textWriter.WriteStartElement("end");
                            textWriter.WriteString(Convert.ToString(xmlEndPosition));
                            textWriter.WriteEndElement();

                            textWriter.WriteEndElement();
                            // TODO zapis konkretnej fazy do XML

                            //zostavajuce(buduce) hodnoty - tie zostanu v poli - 2. cast rozseknutia

                            double xxmlStartPosition = xmlEndPosition;
                            int xxmlDuration = motions4exp.ElementAt(k).Duration + minStartTime - minEndTime;
                            int xxmlStartTime = minEndTime;

                            System.Console.Out.Write("Faza typu vacsia (druha cast)");
                            System.Console.Out.Write("^^" + xxmlStartTime);
                            System.Console.Out.Write("-");
                            System.Console.Out.Write("^^" + motions4exp.ElementAt(k).EndTime);
                            System.Console.Out.Write("\n");

                            motions4exp.ElementAt(k).Duration = xxmlDuration;
                            motions4exp.ElementAt(k).StartTime = xxmlStartTime;
                            motions4exp.ElementAt(k).StartPosition = xxmlStartPosition;
                            done = -1;
                        }
                        System.Console.Out.WriteLine("Pocet nevypisanych pohybov" + motions4exp.Count);

                    }
                    if (done == 0) { k++; }
                } while (k < motions4exp.Count);
                motions4exp.TrimExcess();


                textWriter.WriteEndElement();

                textWriter.WriteStartElement("finalization_phase");
                textWriter.WriteString("DDD");
                textWriter.WriteEndElement();

                textWriter.WriteStartElement("rescue_movement");
                textWriter.WriteString("ROLLBACK");
                textWriter.WriteEndElement();

                textWriter.WriteStartElement("speed_constant");
                textWriter.WriteString(Convert.ToString(Math.Round((double)(1000 * Math.PI * 1000) / (double)(180 * (minEndTime - minStartTime)), 0)));

                textWriter.WriteEndElement();
                textWriter.WriteEndElement();


            } while (motions4exp.Count > 0);

            // Ends the document.

            textWriter.WriteEndDocument();

            // close writer

            textWriter.Close();

            MessageBox.Show("Export pohybov do súboru " + filePath + " bol úspešne ukončený.", "Export do XML",
            MessageBoxButtons.OK, MessageBoxIcon.Information);

            /**/

        }

        public void exportToXMLFileForJim()
        {   //opens the document for writing
            XmlTextWriter textWriter = new XmlTextWriter(filePath, System.Text.Encoding.UTF8);
            textWriter.Formatting = Formatting.Indented;
            string lowSkillName = filePath.Substring(filePath.LastIndexOf("\\") + 1).Substring(0, filePath.Substring(filePath.LastIndexOf("\\") + 1).LastIndexOf("."));
            bool isFinalizePhase = false;
            
            //write starting tag
            textWriter.WriteStartDocument();

            //write main element
            textWriter.WriteStartElement("robot");
            //textWriter.WriteAttributeString("xsi:noNamespaceSchemaLocation", "moves.xsd");
                        
            //write low_skills element
            textWriter.WriteStartElement("low_skills");
            
            //write low_skill element
            textWriter.WriteStartElement("low_skill");
            
            //write name and firstPhase atribute
            textWriter.WriteAttributeString("name", lowSkillName);
            textWriter.WriteAttributeString("firstPhase", lowSkillName + "_phase0");
            
            //close low_skill element
            textWriter.WriteEndElement();
            
            //close low_skills element
            textWriter.WriteEndElement();
            
            //write phases element
            textWriter.WriteStartElement("phases");
            int p = 0;
            do
            {
                // sort pola obsahujuceho vsetky JointMotiony podla zaciatocneho casu
                motions4exp.Sort(delegate(JointMotion jointM1, JointMotion jointM2)
                {
                    return jointM1.StartTime.CompareTo(jointM2.StartTime);
                });
                
                if (motions4exp.Count == 0) break;
                int minEndTime = motions4exp.ElementAt(0).EndTime;
                int minStartTime = motions4exp.ElementAt(0).StartTime;

                for (int i = 0; i < motions4exp.Count; i++)         //cyklus sluziaci na najdenie zaciatku a konca fazy (minStartTime a minEndTime)
                {
                    if (motions4exp.ElementAt(i).StartTime < minEndTime && motions4exp.ElementAt(i).StartTime > minStartTime)
                    {
                        minEndTime = motions4exp.ElementAt(i).StartTime;
                    }
                    if (motions4exp.ElementAt(i).EndTime < minEndTime)
                    {
                        minEndTime = motions4exp.ElementAt(i).EndTime;
                    }
                }
                
                //write phase element
                textWriter.WriteStartElement("phase");
                textWriter.WriteAttributeString("name", lowSkillName + "_phase" + p);
                p++;
                
                //wrire next and isFinal atribute
                if ((motions4exp.ElementAt(0).EndTime == motions4exp.ElementAt(motions4exp.Count - 1).EndTime)
                  &&(motions4exp.ElementAt(0).StartTime == motions4exp.ElementAt(motions4exp.Count - 1).StartTime))
                {
                    textWriter.WriteAttributeString("isFinal", "true");
                    isFinalizePhase = true;
                }
                else
                {
                    textWriter.WriteAttributeString("next", lowSkillName + "_phase" + p);
                    isFinalizePhase = false;
                }                                
                
                //write efectors element
                textWriter.WriteStartElement("effectors");
                
                int k = 0;
                do
                {
                    int done = 0;
                    if (motions4exp.ElementAt(k).StartTime == minStartTime)
                    {
                        if (done == 0 && motions4exp.ElementAt(k).EndTime == minEndTime)
                        {
                            // identifikovana faza (minimalna zaciatocna poloha a minimalna koncova poloha, treba fazu zapisat do XML)

                            // create element
                            textWriter.WriteStartElement(ParseJointName(Convert.ToString(motions4exp.ElementAt(k).Joint)));
                            textWriter.WriteAttributeString("end", Convert.ToString((double)motions4exp.ElementAt(k).EndPosition));
                            textWriter.WriteEndElement();

                            //remove existing elemet from phase
                            motions4exp.Remove(motions4exp.ElementAt(k));

                            done = 1;
                        }
                        if (done == 0 && motions4exp.ElementAt(k).EndTime < minEndTime)
                        {
                            // identifikovana faza (minimalna zaciatocna poloha a minimalna koncova poloha, treba fazu zapisat do XML)
                            
                            // create element
                            textWriter.WriteStartElement(ParseJointName(Convert.ToString(motions4exp.ElementAt(k).Joint)));
                            textWriter.WriteAttributeString("end", Convert.ToString((double)motions4exp.ElementAt(k).EndPosition));
                            textWriter.WriteEndElement();

                            //remove existing elemet from phase
                            motions4exp.Remove(motions4exp.ElementAt(k));

                            done = 1;
                        }
                        if (done == 0 && motions4exp.ElementAt(k).EndTime > minEndTime)
                        {
                            // identifikovana faza (minimalna zaciatocna poloha a minimalna koncova poloha, treba fazu zapisat do XML)

                            //hodnoty na zapisanie (sucasne) - tie sa zapisu do XML - 1. cast rozseknutia
                            int xmlDuration = minEndTime - motions4exp.ElementAt(k).StartTime;
                            int xmlEndTime = minEndTime;
                            double xmlEndPosition = Math.Round((double)xmlDuration / (double)motions4exp.ElementAt(k).Duration * (motions4exp.ElementAt(k).EndPosition - motions4exp.ElementAt(k).StartPosition)) + motions4exp.ElementAt(k).StartPosition;
                            
                            // create element
                            textWriter.WriteStartElement(ParseJointName(Convert.ToString(motions4exp.ElementAt(k).Joint)));
                            textWriter.WriteAttributeString("end", Convert.ToString(xmlEndPosition));
                            textWriter.WriteEndElement();
                                                                                  
                            //zostavajuce(buduce) hodnoty - tie zostanu v poli - 2. cast rozseknutia

                            double xxmlStartPosition = xmlEndPosition;
                            int xxmlDuration = motions4exp.ElementAt(k).Duration + minStartTime - minEndTime;
                            int xxmlStartTime = minEndTime;

                            motions4exp.ElementAt(k).Duration = xxmlDuration;
                            motions4exp.ElementAt(k).StartTime = xxmlStartTime;
                            motions4exp.ElementAt(k).StartPosition = xxmlStartPosition;
                            done = -1;
                        }
                    }
                    if (done == 0) { k++; }
                } while (k < motions4exp.Count);
              
                motions4exp.TrimExcess();

                //close efectors element
                textWriter.WriteEndElement();
                
                //duration element
                textWriter.WriteStartElement("duration");
                textWriter.WriteString((minEndTime - minStartTime).ToString());
                textWriter.WriteEndElement();
                
                //finalize element
                if (isFinalizePhase)
                {
                    textWriter.WriteStartElement("finalize");
                    textWriter.WriteString(lowSkillName + "_phase" + (p - 1).ToString());
                    textWriter.WriteEndElement();
                }

                //close phase element
                textWriter.WriteEndElement();
            } while (motions4exp.Count > 0);

            //close phases element
            textWriter.WriteEndElement();
            
            //ends robot element
            textWriter.WriteEndElement();

            // Ends the document.
            textWriter.WriteEndDocument();

            // close writer and save to file
            textWriter.Close();

            MessageBox.Show("Export pohybov do súboru " + filePath + " bol úspešne ukončený.", "Export do XML",
            MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private String ParseJointName(String jointName)
        {
            switch (jointName)
            {


                case "Torso":
                    return "t";

                case "Head":
                    return "he2";
                case "Neck":
                    return "he1";
                case "LeftShoulder":
                    return "lae1";
                case "RightShoulder":
                    return "rae1";
                case "LeftUpperArm":
                    return "lae2";
                case "RightUpperArm":
                    return "rae2";
                case "LeftElbow":
                    return "lae3";
                case "RightElbow":
                    return "rae3";
                case "LeftLowerArm":
                    return "lae4";
                case "RightLowerArm":
                    return "rae4";
                case "LeftHip1":
                    return "lle1";
                case "RightHip1":
                    return "rle1";
                case "LeftHip2":
                    return "lle2";
                case "RightHip2":
                    return "rle2";
                case "LeftThigh":
                    return "lle3";
                case "RightThigh":
                    return "rle3";
                case "LeftShank":
                    return "lle4";
                case "RightShank":
                    return "rle4";
                case "LeftAnkle":
                    return "lle5";
                case "RightAnkle":
                    return "rle5";
                case "LeftFoot":
                    return "lle6";
                case "RightFoot":
                    return "rle6";
                default:
                    return "error";

            }

            // return parsedJointName;
        }

    }
}
