﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using RobotBehaviourEditor.Model;
using RobotBehaviourEditor.ActionEditor;
using RobotBehaviourEditor.Formats;

namespace RobotBehaviourEditor.Library
{
    public partial class BrowserForm : Form
    {
        public BrowserForm()
        {
            InitializeComponent();            

            Library.LibraryChanged += new Library.LibraryChangedHandler(Library_LibraryChanged);
        }

        public void Library_LibraryChanged(object sender, EventArgs e)
        {
            BindContent();
        }

        private void BindContent()
        {
            if (libraryTree.IsDisposed) return;

            libraryTree.Nodes.Clear();

            TreeNode topNode = new TreeNode("Knižnica");
            topNode.ImageIndex = 0;
            TreeNode motionNode = topNode.Nodes.Add("Pohyby");
            motionNode.ImageIndex = 1;
            AddMotions(motionNode);

            TreeNode intensionNode = topNode.Nodes.Add("Zámery");
            intensionNode.ImageIndex = 1;
            TreeNode preconditionNode = topNode.Nodes.Add("Predpodmienky");
            preconditionNode.ImageIndex = 1;

            TreeNode serverMessagesNode = topNode.Nodes.Add("Správy zo servera");
            serverMessagesNode.ImageIndex = 1;
            AddServerMessages(serverMessagesNode);

            TreeNode actionsNode = topNode.Nodes.Add("Akcie");
            actionsNode.ImageIndex = 1;
            //AddActions(serverMessagesNode);

            libraryTree.Nodes.Add(topNode);

            motionNode.EnsureVisible();
        }

        private void AddServerMessages(TreeNode serverMessagesNode)
        {
            try
            {
                if (Directory.Exists( Program.Config.LibraryPath))
                {
                    string dir =Path.Combine(Program.Config.LibraryPath, "ServerMessages");
                    if (!Directory.Exists(dir))
                        Directory.CreateDirectory(dir);
                }

                string[] files = Directory.GetFiles(Path.Combine(Program.Config.LibraryPath, "ServerMessages"));

                foreach (string file in files)
                {
                    TreeNode newNode = new TreeNode();
                    FileInfo info = new FileInfo(file);                    

                    newNode.ImageIndex = 0;
                    newNode.Text = info.Name;
                    newNode.Tag = new ServerMessageAction(file, this.MdiParent);

                    serverMessagesNode.Nodes.Add(newNode);
                    //motionNode.Nodes.Add(info.Name);
                }
            }
            catch (DirectoryNotFoundException)
            {
                MessageBox.Show("Adresár s knižnicou sa nenašiel. Skontrolujte definovanú cestu ku knižnici v nastaveniach aplikácie.", null, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Pri načítaní serverových správ nastala neočakávaná chyba.\n\n" + ex.ToString(), null, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void AddMotions(TreeNode motionNode)
        {
            try
            {
                string[] files = Directory.GetFiles(Path.Combine(Program.Config.LibraryPath, "Motions"));

                foreach (string file in files)
                {
                    TreeNode newNode = new TreeNode();
                    FileInfo info = new FileInfo(file);
                    RobotMotion motion = RobotMotion.Load(file);

                    newNode.ImageIndex = 2;
                    newNode.Text = motion.Name + " (" + info.Name + ")";
                    newNode.ToolTipText = motion.Description;
                    newNode.Tag = new MotionAction(file, this.MdiParent);

                    motionNode.Nodes.Add(newNode);
                    //motionNode.Nodes.Add(info.Name);
                }
            }
            catch (DirectoryNotFoundException)
            {
                MessageBox.Show("Adresár s knižnicou pohybov sa nenašiel. Skontrolujte definovanú cestu ku knižnici v nastaveniach aplikácie.", null, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Pri načítaní pohybov nastala neočakávaná chyba.\n\n" + ex.ToString(), null, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void libraryTree_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (libraryTree.SelectedNode.Tag != null)
            {
                (libraryTree.SelectedNode.Tag as IAction).DoAction();                
            }
        }

        private void importServerMessageButton_Click(object sender, EventArgs e)
        {
            Commands.ImportServerMessageFile();
        }

        private void newMotionButton_Click(object sender, EventArgs e)
        {
            Commands.CreateNewMotion(this.MdiParent);
        }

        private void BrowserForm_Load(object sender, EventArgs e)
        {
            BindContent();
        }

        private void newActionButton_Click(object sender, EventArgs e)
        {
            ActionEditorForm frm = new ActionEditorForm();
            frm.MdiParent = this.MdiParent;
            frm.Show();
        }

        private void importXmlButton_Click(object sender, EventArgs e)
        {
            Commands.ImportXmlMessageFile();
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            ServerLogImportOptions serverLogImportOptions = new ServerLogImportOptions();
            serverLogImportOptions.Visible = true;
        } 
    }
}
