﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Xml;
using RobotBehaviourEditor.Formats;

namespace RobotBehaviourEditor.Library
{
    public static class Library
    {
        public delegate void LibraryChangedHandler(object sender, EventArgs e);
        public static event LibraryChangedHandler LibraryChanged;

        public const string XML_DREAM_TEAM_PLAYER = "xml_dream_team_player";

        public const string XML_JIM_PLAYER = "xml_jim_player";

        public static void OnLibraryChanged(EventArgs e)
        {
            if (LibraryChanged != null) LibraryChanged(null, e);
        }

        public static void ImportMotion(string motionFileName)
        {
            try
            {
                string name = new FileInfo(motionFileName).Name;
                string motionDir = Path.Combine(Program.Config.LibraryPath, "Motions");
                string destination = Path.Combine(motionDir, name);

                if (!Directory.Exists(motionDir))
                    Directory.CreateDirectory(motionDir);

                if (File.Exists(destination))                
                {
                    if (MessageBox.Show("Pohyb s rovnakým názvom už v knižnici existuje. Chcete ho nahradiť?", "Import", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                        return;
                }

                File.Copy(motionFileName, destination, true);

                OnLibraryChanged(new EventArgs());
            }            
            catch (Exception ex)
            {
                MessageBox.Show("Pri importovaní pohybu nastala neočakávaná chyba.\n" + ex.Message, "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public static void ImportServerMessages(string serverMessagesFileName)
        {
            try
            {
                string name = new FileInfo(serverMessagesFileName).Name;
                string serverMessagesDir = Path.Combine(Program.Config.LibraryPath, "ServerMessages");
                string destination = Path.Combine(serverMessagesDir, name);

                if (!Directory.Exists(serverMessagesDir))
                    Directory.CreateDirectory(serverMessagesDir);

                File.Copy(serverMessagesFileName, destination);

                OnLibraryChanged(new EventArgs());
            }
            catch (Exception ex)
            {
                MessageBox.Show("Pri importovaní správ zo servera nastala neočakávaná chyba.\n" + ex.Message, "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public static void ImportMovesFromXml(string xmlFilePath, string xmlType)
        {
            try
            {
                IXmlImporter importer=null;

                if (xmlType.Equals(Library.XML_JIM_PLAYER))
                {
                    importer = new XmlImporterForJim(xmlFilePath);
                }
                else if (xmlType.Equals(Library.XML_DREAM_TEAM_PLAYER))
                {
                    importer = new XmlImporter(xmlFilePath);
                }
                                
                ImportXmlLowSkillPicker window = new ImportXmlLowSkillPicker(importer);
                window.Visible = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public static void Update()
        {
            OnLibraryChanged(new EventArgs());
        }
    }
}
