﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace RobotBehaviourEditor.Model
{
    public class JointMotion : IMotionItem
    {        
        #region Properties
        JointTypes joint;
        public JointTypes Joint
        {
            get { return joint; }
            set { joint = value; }
        }

        double startPosition;
        public double StartPosition
        {
            get { return startPosition; }
            set { startPosition = value; }
        }

        double endPosition;
        public double EndPosition
        {
            get { return endPosition; }
            set { endPosition = value; }
        }

        int startTime;
        public int StartTime
        {
            get { return startTime; }
            set { startTime = value; }
        }

        int duration;
        public int Duration
        {
            get { return duration; }
            set { duration = value; }
        }

        public int EndTime
        {
            get { return startTime + duration; }            
        }

        bool isActive = true;
        public bool IsActive
        {
            get { return isActive; }
            set { isActive = value; }
        }

        List<JointCondition> preConditions = new List<JointCondition>();
        public List<JointCondition> PreConditions
        {
            get { return preConditions; }
            set { preConditions = value; }
        }
        #endregion

        #region Constructors
        public JointMotion()
        {
        }

        public JointMotion(JointMotion motion)
        {
            this.duration = motion.duration;
            this.endPosition = motion.endPosition;
            this.joint = motion.joint;
            //this.preConditions = new List<JointCondition>(motion.preConditions);
            this.startPosition = motion.startPosition;
            this.startTime = motion.startTime;
        }

        public JointMotion(JointTypes joint, double startPosition, double endPosition, int duration, int startTime, List<JointCondition> preConditions)
        {
            this.joint = joint;
            this.startPosition = startPosition;
            this.endPosition = endPosition;
            this.duration = duration;
            this.startTime = startTime;
            this.preConditions = preConditions;
        }
        #endregion

        // Prida podmienku, ktora musi platit pred zacatim pohybu
        public void AddPreCondition(JointCondition condition)
        {
            preConditions.Add(condition);
        }

        public override string ToString()
        {
            return Joints.GetJointByType(joint).Name + ": " + startPosition + "° - " + endPosition + "° (" + duration + " ms)";
        }

        public double GetJoinPositionForTime(double timeOffset)
        {
            double coef = (timeOffset - StartTime) / duration;
            return startPosition + (endPosition - startPosition) * coef;
        }
    }
}
