﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace RobotBehaviourEditor.Model
{
    public static class Joints
    {
        private readonly static JointItem[] joints = new JointItem[] 
            {
                new JointItem(JointTypes.Neck, "HE1", "Krk", -120, 120, Color.FromArgb(102,153, 255)),
                new JointItem(JointTypes.Head, "HE2", "Hlava", -45, 45, Color.FromArgb(50,76, 127)),
                new JointItem(JointTypes.LeftShoulder, "LAE1", "Ľavé plece", -120, 120, Color.FromArgb(51,204,153)),
                new JointItem(JointTypes.RightShoulder, "RAE1", "Pravé plece", -120, 120, Color.FromArgb(51,204,153)),
                new JointItem(JointTypes.LeftUpperArm, "LAE2", "Ľavé rameno",  -1, 95, Color.FromArgb(25,101,76)),
                new JointItem(JointTypes.RightUpperArm, "RAE2", "Pravé rameno", -95, 1, Color.FromArgb(25,101,76)),
                new JointItem(JointTypes.LeftElbow, "LAE3", "Ľavý lakeť", -120, 120, Color.FromArgb(51,204,51)),
                new JointItem(JointTypes.RightElbow, "RAE3", "Pravý lakeť", -120, 120, Color.FromArgb(51,204,51)),
                new JointItem(JointTypes.LeftLowerArm, "LAE4", "Ľavé predlaktie", -90, 1, Color.FromArgb(25,101,25)),
                new JointItem(JointTypes.RightLowerArm, "RAE4", "Pravé predlaktie", -1, 90, Color.FromArgb(25,101,25)),
                new JointItem(JointTypes.LeftHip1, "LLE1", "Ľavý bok", -90, 1, Color.FromArgb(153,255,0)),
                new JointItem(JointTypes.RightHip1, "RLE1", "Pravý bok", -90, 1, Color.FromArgb(153,255,0)),
                new JointItem(JointTypes.LeftHip2, "LLE2", "Ľavý bok 2", -25, 45, Color.FromArgb(76,127,0)),
                new JointItem(JointTypes.RightHip2, "RLE2", "Pravý bok 2", -45, 25, Color.FromArgb(76,127,0)),
                new JointItem(JointTypes.LeftThigh, "LLE3", "Ľavé stehno", -25, 100, Color.FromArgb(255,204,0)),
                new JointItem(JointTypes.RightThigh, "RLE3", "Pravé stehno", -25, 100, Color.FromArgb(255,205,0)),
                new JointItem(JointTypes.LeftShank, "LLE4", "Ľavé lýtko", -130, 1, Color.FromArgb(255, 102, 0)),
                new JointItem(JointTypes.RightShank, "RLE4", "Pravé lýtko", -130, 1, Color.FromArgb(255,102,0)),
                new JointItem(JointTypes.LeftAnkle, "LLE5", "Ľavý členok", -45, 75, Color.FromArgb(204,51,0)),
                new JointItem(JointTypes.RightAnkle, "RLE5", "Pravý členok", -45, 75, Color.FromArgb(204,51,0)), 
                new JointItem(JointTypes.LeftFoot, "LLE6", "Ľavé chodidlo", -45, 25, Color.FromArgb(153,51,102)),
                new JointItem(JointTypes.RightFoot, "RLE6", "Pravé chodidlo", -25, 45, Color.FromArgb(153,51,102)),
                new JointItem(JointTypes.Torso, "T", "Torzo", 0, 0, Color.Black)
            };

        public static JointItem[] JointItems
        {
            get { return joints; }
        }

        public static JointItem GetJointByType(JointTypes type)
        {
            foreach (JointItem joint in joints)
            {
                if (joint.Type == type)
                    return joint;
            }
            return null;
        }

        public static JointTypes GetJointTypeById(string id)
        {
            string nid = id.Trim().ToUpper();
            foreach (JointItem joint in joints)
            {
                if (joint.Id == nid)
                    return joint.Type;
            }
            return JointTypes.Head;
        }
    }
}
