﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Media3D;

namespace RobotBehaviourEditor.Model.Physics
{
    /// <summary>
    /// Obsahuje kompletnu zakladnu konfiguraciu hraca
    /// </summary>
    static class InitialConfiguration
    {
        /************************************************/
        /*                 Torso                        */
        /************************************************/
        public const double TorsoMass = 1.2171f;

        /************************************************/
        /*                    Arms                      */
        /************************************************/

        //shoulder
        public static readonly Vector3D RightShoulderTranslation = new Vector3D(0.098f, 0, 0.075f);
        public static readonly Vector3D LeftShoulderTranslation = new Vector3D(-0.098f, 0, 0.075f);
        public const double ShoulderMass = 0.07f;
        public static readonly Vector3D ShoulderAnchor = new Vector3D(0, 0, 0);
        public static readonly Vector3D ShoulderAxis = new Vector3D(1, 0, 0);

        //upper arm
        public static readonly Vector3D RightUpperarmTranslation = new Vector3D(0.01f, 0.02f, 0);
        public static readonly Vector3D LeftUpperarmTranslation = new Vector3D(-0.01f, 0.02f, 0);
        public const double UpperarmMass = 0.15f;
        public static readonly Vector3D UpperarmAnchor = new Vector3D(0, 0, 0);
        public static readonly Vector3D UpperarmAxis = new Vector3D(0, 0, 1);

        //elbow
        public static readonly Vector3D RightElbowTranslation = new Vector3D(-0.01f, 0.07f, 0.009f);
        public static readonly Vector3D LeftElbowTranslation = new Vector3D(0.01f, 0.07f, 0.009f);
        public const double ElbowMass = 0.035f;
        public static readonly Vector3D ElbowAnchor = new Vector3D(0, 0, 0);
        public static readonly Vector3D ElbowAxis = new Vector3D(0, 1, 0);

        //lower arm
        public static readonly Vector3D LowerarmTranslation = new Vector3D(0, 0.05f, 0);
        public const double LowerarmMass = 0.2f;
        public static readonly Vector3D LowerarmAnchor = new Vector3D(0, -0.05f, 0);
        public static readonly Vector3D LowerarmAxis = new Vector3D(0, 0, 1);
        const double LowerarmSizeX = 0.05;
        const double LowerarmSizeY = 0.11;
        const double LowerarmSizeZ = 0.05;
        public static readonly Point3D[] LowerarmFrontalPoints =
        {
            new Point3D(LowerarmSizeX / 2, LowerarmSizeY / 2, LowerarmSizeZ / 2),
            new Point3D(-1 * LowerarmSizeX / 2, LowerarmSizeY / 2, LowerarmSizeZ / 2),
            new Point3D(LowerarmSizeX / 2, LowerarmSizeY / 2, -1 * LowerarmSizeZ / 2),
            new Point3D(-1 * LowerarmSizeX / 2, LowerarmSizeY / 2, -1 * LowerarmSizeZ / 2),
        };

        /************************************************/
        /*                   Legs                       */
        /************************************************/

        //hip1
        public static readonly Vector3D RightHip1Translation = new Vector3D(0.055f, -0.01f,-0.115f);
        public static readonly Vector3D LeftHip1Translation = new Vector3D(-0.055f, -0.01f, -0.115f);
        public const double Hip1Mass = 0.09f;
        public static readonly Vector3D Hip1Anchor = new Vector3D(0, 0, 0);
        public static readonly Vector3D RightHip1Axis = new Vector3D(-0.7071f, 0, 0.7071f);
        public static readonly Vector3D LeftHip1Axis = new Vector3D(-0.7071f, 0, -0.7071f);

        //hip2
        public static readonly Vector3D Hip2Translation = new Vector3D(0, 0, 0);
        public const double Hip2Mass = 0.125f;
        public static readonly Vector3D Hip2Anchor = new Vector3D(0, 0, 0);
        public static readonly Vector3D Hip2Axis = new Vector3D(0, 1, 0);

        //thigh
        public static readonly Vector3D ThighTranslation = new Vector3D(0, 0.01f, -0.04f);
        public const double ThighMass = 0.275f;
        public static readonly Vector3D ThighAnchor = new Vector3D(0, -0.01f, 0.04f);
        public static readonly Vector3D ThighAxis = new Vector3D(1, 0, 0);

        //shank
        public static readonly Vector3D ShankTranslation = new Vector3D(0, 0.005f, -0.125f);
        public const double ShankMass = 0.225f;
        public static readonly Vector3D ShankAnchor = new Vector3D(0, -0.01f, 0.045f);
        public static readonly Vector3D ShankAxis = new Vector3D(1, 0, 0);
        
        //ankle
        public static readonly Vector3D AnkleTranslation = new Vector3D(0, -0.01f, -0.055f);
        public const double AnkleMass = 0.125f;
        public static readonly Vector3D AnkleAnchor = new Vector3D(0, 0, 0);
        public static readonly Vector3D AnkleAxis = new Vector3D(1, 0, 0);

        //foot
        public static readonly Vector3D FootTranslation = new Vector3D(0, 0.03f, -0.035f);
        public const double FootMass = 0.2f;
        public static readonly Vector3D FootAnchor = new Vector3D(0, -0.03f, 0.035f);
        public static readonly Vector3D FootAxis = new Vector3D(0, 1, 0);
        const double FootSizeX = 0.08;
        const double FootSizeY = 0.16;
        const double FootSizeZ = 0.03;
        public static readonly Point3D[] FootFrontalPoints =
        {
            new Point3D(FootSizeX / 2, FootSizeY / 2, -1 * FootSizeZ / 2),
            new Point3D(-1 * FootSizeX / 2, FootSizeY / 2, -1 * FootSizeZ / 2),
            new Point3D(FootSizeX / 2, -1 * FootSizeY / 2, -1 * FootSizeZ / 2),
            new Point3D(-1 * FootSizeX / 2, -1 * FootSizeY / 2, -1 * FootSizeZ / 2),
        };

        /************************************************/
        /*                   Head                       */
        /************************************************/

        //neck
        public static readonly Vector3D NeckTranslation = new Vector3D(0, 0, 0.09f);
        public const double NeckMass = 0.05f;
        public static readonly Vector3D NeckAnchor = new Vector3D(0, 0, 0);
        public static readonly Vector3D NeckAxis = new Vector3D(0, 0, 1);

        //head
        public static readonly Vector3D HeadTranslation = new Vector3D(0, 0, 0.065f);
        public const double HeadMass = 0.35f;
        public static readonly Vector3D HeadAnchor = new Vector3D(0, 0, -0.005f);
        public static readonly Vector3D HeadAxis = new Vector3D(1, 0, 0);
    }
}
