﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Media3D;
using System.Windows;

namespace RobotBehaviourEditor.Model.Physics
{
    static class MathHelper
    {
        public static Vector3D GetInverted(this Vector3D vector)
        {
            vector.Negate();
            return vector;
        }

        public static Point3D GetDestinationPoint(this Vector3D vector)
        {
            return new Point3D(vector.X, vector.Y, vector.Z);
        }

        /// <summary>
        /// posunie bod o nejaky vektor
        /// </summary>
        public static Point3D GetTranslated(this Point3D point, Vector3D vector)
        {
            point.Offset(vector.X, vector.Y, vector.Z);
            return point;
        }

        public static Vector3D GetVector(this Point3D point)
        {
            return new Vector3D(point.X, point.Y, point.Z);
        }

        /// <summary>
        /// rotuje bod okolo zadaneho vektora o zadany uhol
        /// funguje pre vektory otocene v smere jednej z hlavnych osi
        /// </summary>
        public static Point3D GetRotated(this Point3D point, Vector3D vector, double angle)
        {
            //toto je hadam najdolezitejsia metoda
            //otocenie bodu okolo vektoru o nejaky uhol
            //samotny vektor udava smer aj orientaciu otacania

            //ak je uhol 0, nema zmysel vykonavat rotaciu]
            //ak je bod [0,0,0] tiez ho nema zmysel rotovat
            if (angle == 0 || point.Equals(new Point3D(0, 0, 0)))
                return point;

            // normalizacia vektora
            Vector3D vector_normal = new Vector3D();
            vector_normal.X = vector.X / Math.Sqrt(vector.X * vector.X + vector.Y * vector.Y + vector.Z * vector.Z);
            vector_normal.Y = vector.Y / Math.Sqrt(vector.X * vector.X + vector.Y * vector.Y + vector.Z * vector.Z);
            vector_normal.Z = vector.Z / Math.Sqrt(vector.X * vector.X + vector.Y * vector.Y + vector.Z * vector.Z);

            vector = vector_normal;

            //prepocet na radiany
            angle = angle * 2 * Math.PI / 360;

            // rotacia bodu okolo vektora
            // pouziva sa Rodriguezova metoda

            // vypocitame rotacnu maticu
            double[,] rMatrix = new double[3, 3];
            double cos_a = Math.Cos(angle);
            double sin_a = Math.Sin(angle);

            rMatrix[0, 0] = round(cos_a + vector.X * vector.X * (1 - cos_a));
            rMatrix[1, 0] = round(vector.Z * sin_a + vector.X * vector.Y * (1 - cos_a));
            rMatrix[2, 0] = round(-vector.Y * sin_a + vector.X * vector.Z * (1 - cos_a));

            rMatrix[0, 1] = round(vector.X * vector.Y * (1 - cos_a) - vector.Z * sin_a);
            rMatrix[1, 1] = round(cos_a + vector.Y * vector.Y * (1 - cos_a));
            rMatrix[2, 1] = round(vector.X * sin_a + vector.Y * vector.Z * (1 - cos_a));

            rMatrix[0, 2] = round(vector.Y * sin_a + vector.X * vector.Z * (1 - cos_a));
            rMatrix[1, 2] = round(-vector.X * sin_a + vector.Y * vector.Z * (1 - cos_a));
            rMatrix[2, 2] = round(cos_a + vector.Z * vector.Z * (1 - cos_a));

            // vynasobime rotacnu maticu a aktualne suradnice bodu,
            // dostaneme zrotovanu polohu o uhol angle

            double x, y, z;
            x = round(rMatrix[0, 0] * point.X + rMatrix[0, 1] * point.Y + rMatrix[0, 2] * point.Z);
            y = round(rMatrix[1, 0] * point.X + rMatrix[1, 1] * point.Y + rMatrix[1, 2] * point.Z);
            z = round(rMatrix[2, 0] * point.X + rMatrix[2, 1] * point.Y + rMatrix[2, 2] * point.Z);

            return new Point3D(x, y, z);
        }

        /// <summary>
        /// odstranuje zaokruhlovacie chyby pri vypoctoch
        /// </summary>
        private static double round(double d)
        {

            int integer_part = (int)Math.Round(d);

            double difference = Math.Abs(d) - Math.Abs(integer_part);

            if (difference < 0.000001 && difference > -0.000001)
                return (double)integer_part;
            else
                return d;

        }

        /// <summary>
        /// vypocita uhol v kladnom pravotocivom smere od vektora 1 k vektoru 2
        /// </summary>
        /// <returns></returns>
        public static double GetClockwiseAngleTo(this Vector vector1, Vector vector2)
        {
            double
                angle1 = GetAngleFromAxisX(vector1),
                angle2 = GetAngleFromAxisX(vector2),
                angle = angle1 - angle2;

            if (angle < 0)
                angle = 2 * Math.PI + angle;

            return angle;
        }

        /// <summary>
        /// vypocita uhol vektora od x-ovej osi
        /// v zapornom lavotocivom smere,
        /// podla jednotkovej kruznice
        /// </summary>
        /// <returns>uhol v radianoch</returns>
        private static double GetAngleFromAxisX(Vector vector)
        {
            vector.Normalize();
            double angle = Math.Atan2(vector.Y, vector.X);

            if (angle < 0)
                angle += 2 * Math.PI;

            return angle;
        }
    }
}
