﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace RobotBehaviourEditor.Model
{
    public class State
    {
        public State()
        {
        }

        public State(string name)
        {
            this.name = name;
        }

        public State(State state)
        {
            this.name = state.name;
            this.synchronized = state.synchronized;
            this.motions = new List<JointMotion>();
            for (int i = 0; i < state.motions.Count; i++)
            {
                this.motions.Add(new JointMotion(state.motions[i]));
            }
        }

        private List<JointMotion> motions = new List<JointMotion>();
        public ReadOnlyCollection<JointMotion> Motions
        {
            get { return motions.AsReadOnly(); }            
        }

        private string name;
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        private bool synchronized;
        public bool Synchronized
        {
            get { return synchronized; }
            set { synchronized = value; }
        }

        public void AddMotion(JointMotion motion)
        {
            motions.Add(motion);
        }

        public void AddMotions(JointMotion[] motions)
        {
            this.motions.AddRange(motions);
        }

        public override string ToString()
        {
            return Name + " (" + motions.Count + " pohybov)";
        }

        public void OrderMotionsByTime()
        {
            motions = motions.OrderBy(x => x.StartTime).ToList();
        }
    }
}
