﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using RobotBehaviourEditor.Model;
using RobotBehaviourEditor.MotionEditor.Visualisation;

namespace RobotBehaviourEditor.MotionEditor
{
    public partial class JointMotionEditor : Form
    {
        private JointMotion result;
        public JointMotion Result
        {
            get { return result; }
            private set { result = value; }
        }

        private List<JointMotion> jointMotions = new List<JointMotion>();
        public List<JointMotion> JointMotions
        {
            get { return jointMotions; }
            set { jointMotions = value; }
        }

        public JointMotionEditor(IList<JointMotion> jointMotions)
        {
            InitializeComponent();

            InitializeJointCombo();

            JointMotions.Clear();
            JointMotions.AddRange(jointMotions);
        }

        public JointMotionEditor(JointMotion editedMotion, IList<JointMotion> jointMotions) : this(jointMotions)
        {
            try
            {
                for (int i = 0; i < jointCombo.Items.Count; i++)
                {
                    if (((JointItem)jointCombo.Items[i]).Type == editedMotion.Joint)
                    {
                        jointCombo.SelectedIndex = i;
                        jointCombo_SelectedIndexChanged(this, new EventArgs());
                        break;
                    }
                }

                initialAngleUpDown.Value = (decimal)editedMotion.StartPosition;
                finalAngleUpDown.Value = (decimal)editedMotion.EndPosition;
                timeUpDown.Value = editedMotion.StartTime;
                moveDurationUpDown.Value = editedMotion.Duration;
            }
            catch
            {
            }
        }   

        private void addButton_Click(object sender, EventArgs e)
        {           
            JointItem item = ((JointItem)jointCombo.SelectedItem);

            if (item != null)
            {
                result = new JointMotion(item.Type,
                                  (double)initialAngleUpDown.Value,
                                  (double)finalAngleUpDown.Value,
                                  (int)moveDurationUpDown.Value,
                                  (int)timeUpDown.Value,
                                  conditionList.Items.Cast<JointCondition>().ToList()
                                  );
            }
            else
            {
                MessageBox.Show("Pohyb kĺbu nie je možné pridať. Nevybrali ste kĺb.", "Pohyb kĺbu", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.None;
            }
        }

        private void InitializeJointCombo()
        {
            jointCombo.Items.AddRange(Joints.JointItems);

        }

        private void jointCombo_SelectedIndexChanged(object sender, EventArgs e)
        {

            finalAngleUpDown.Minimum = ((JointItem)jointCombo.SelectedItem).MinValue;
            finalAngleUpDown.Maximum = ((JointItem)jointCombo.SelectedItem).MaxValue;
        }

        private void addConditionButton_Click(object sender, EventArgs e)
        {
            ConditionForm frm = new ConditionForm();
            frm.ShowDialog(this);
            if (frm.DialogResult == DialogResult.OK)
            {
                conditionList.Items.Add(frm.Result);
            }
        }

        private void initConditionsRadio_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void timeRadio_CheckedChanged(object sender, EventArgs e)
        {
            conditionsPanel.Enabled = initConditionsRadio.Checked;
            timeUpDown.Enabled = timeRadio.Checked;
        }

        private void speedUpDown_ValueChanged(object sender, EventArgs e)
        {
            UpdateSpeed();
        }

        private void initialPositionUpDown_ValueChanged(object sender, EventArgs e)
        {
            UpdateSpeed();
        }

        private void finalPositionUpDown_ValueChanged(object sender, EventArgs e)
        {
            UpdateSpeed();
        }

        private void UpdateSpeed()
        {
            if (speedUpDown.Value != 0)
            {
                int value = (int)Math.Max(Math.Abs(finalAngleUpDown.Value - initialAngleUpDown.Value) / speedUpDown.Value * 1000, 0);
                if (value > moveDurationUpDown.Minimum && value < moveDurationUpDown.Maximum)
                    moveDurationUpDown.Value = value;
            }
        }        

        private void moveDurationUpDown_ValueChanged(object sender, EventArgs e)
        {
            if (moveDurationUpDown.Value != 0)
            {
                speedUpDown.Value = Math.Abs(finalAngleUpDown.Value - initialAngleUpDown.Value) / (moveDurationUpDown.Value / 1000);
            }
        }

        private void initialAngleButton_Click(object sender, EventArgs e)
        {
            AngleForm frm = new AngleForm();
            Point pt = initialAngleButton.PointToScreen(new Point(0, 0));
            frm.Left = pt.X;
            frm.Top = pt.Y + initialAngleButton.Height;
            frm.Show();            
        }

        private void robotButton_Click(object sender, EventArgs e)
        {
            Robot3DForm frm = new Robot3DForm();
            frm.ShowDialog(this);

            //var window = new System.Windows.Window();
            //window.Content = new RobotBehaviourEditor.MotionEditor.Visualisation.Robot3DControl(window)
            //    {
            //        Width = window.Width,
            //        Height = window.Height,
            //    };            
            //window.ShowDialog();
            
            /*
            RobotForm frm = new RobotForm();
            frm.ShowDialog();

            if (frm.SelectedPart != null)
            {
                for (int i = 0; i < jointCombo.Items.Count - 1; i++)
                {
                    if (((JointItem)jointCombo.Items[i]).Type == frm.SelectedPart.Type)
                    {
                        jointCombo.SelectedIndex = i;
                    }
                }
            }
            */
        }

        private void timeUpDown_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                int maxTime = -1;
                int time = (int)timeUpDown.Value;
                int duration = (int)moveDurationUpDown.Value;
                double endPosition = 0;
                bool wrongStartTime = false;

                foreach (var motion in jointMotions)
                {                    
                    if (motion.Joint == ((JointItem)jointCombo.SelectedItem).Type)
                    {
                        if (motion.StartTime < time + duration && time < motion.StartTime + motion.Duration)
                        {
                            wrongStartTime = true;
                        }
                        else if (motion.StartTime + motion.Duration <= time && motion.StartTime > maxTime)
                        {
                            maxTime = motion.StartTime;
                            endPosition = motion.EndPosition;
                        }
                    }
                }

                initialAngleUpDown.Value = (decimal)endPosition;

                if (wrongStartTime)
                {
                    timeUpDown.BackColor = Color.Red;
                }
                else
                {
                    timeUpDown.BackColor = SystemColors.Window;
                }
            }
            catch
            {
            }
        }
    }
}
