﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using RobotBehaviourEditor.Model;
using System.IO;
using System.Drawing.Drawing2D;
using RobotBehaviourEditor.MotionEditor.Configuration;
using System.Diagnostics;
using RobotBehaviourEditor.MotionEditor;
using RobotBehaviourEditor.MotionEditor.Visualisation;
using RobotBehaviourEditor.Formats;

namespace RobotBehaviourEditor.MotionEditor
{
    public partial class MotionEditorForm : Form
    {
        #region Fields
        Robot3DControl robot3DControl;               

        private string windowMainTitle = "Editor pohybov";

        int stateCounter = 0;
        bool stateChanged = false;
        int openStateIndex = -1;
        RobotBehaviourEditor.Model.RobotMotion motion = new RobotBehaviourEditor.Model.RobotMotion();        

        Process serverProcess;
        Process agentProcess;

        private RunSettings savedRunSettings = new RunSettings();
        #endregion
        public const string XML_CRITICAL_ERROR_PLAYER = "xml_critical_error_player";

        public const string XML_JIM_PLAYER = "xml_jim_player";

        #region Proberties
        private string openFileName;
        public string OpenFileName
        {
            get { return openFileName; }
        }

        public bool StateChanged
        {
            get { return stateChanged; }
            set 
            { 
                stateChanged = value;                
            }
        }
        #endregion

        #region Constructors
        public MotionEditorForm()
        {
            InitializeComponent();

            this.Text = "Nový pohyb - " + windowMainTitle;            
        }

        public MotionEditorForm(RobotMotion motion)
        {
            this.motion = motion;
            InitializeComponent();
            UpdateStates();
            Refresh();

            this.Text = "Nový pohyb - " + windowMainTitle;
        }

        public MotionEditorForm(string fileName)
            : this()
        {
            LoadMotion(fileName);

            this.Text = motion.Name + " - " + windowMainTitle;
        }
        #endregion

        public static MotionEditorForm OpenMotion(string fileName, Form mdiParent)
        {
            MotionEditorForm frm = new MotionEditorForm(fileName);
            frm.MdiParent = mdiParent;
            frm.Show();
            return frm;
        }

        private void LoadMotion(string fileName)
        {
            RobotMotion openedMotion = RobotMotion.Load(fileName);
            this.motion = openedMotion;
            openFileName = fileName;

            UpdateStates();
            Refresh();
        }
       
        private void addMoveButton_Click(object sender, EventArgs e)
        {
            JointMotionEditor frm = new JointMotionEditor(motionDiagram.JointMotions);
            frm.ShowDialog();

            if (frm.DialogResult == DialogResult.OK && frm.Result != null)
            {
                motionDiagram.AddJoinMotion(frm.Result);
                StateChanged = true;
            }
        }

        private void addStateButton_Click(object sender, EventArgs e)
        {
            stateCounter++;

            motion.AddState(new State("Stav #" + stateCounter));
            UpdateStates();
        }

        private void UpdateStates()
        {
            int selIndex = stateList.SelectedIndex;
            stateList.Items.Clear();
            stateList.Items.AddRange(motion.States.ToArray());
            stateList.SelectedIndex = selIndex;

            //nextStateCombo.Items.Clear();
            //nextStateCombo.Items.AddRange(action.States.ToArray());            
        }

        private void stateList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (stateChanged) //MessageBox.Show("Neuložili ste stav. Chcete ho uložiť teraz?", this.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes
            {
                SaveCurrentState();
            }

            if (stateList.SelectedIndex != -1)
            {
                stateTabControl.Visible = true;
                openStateIndex = stateList.SelectedIndex;
                BindState(motion.States[openStateIndex]);                
            }
            else
            {
                stateTabControl.Visible = false;
            }            
        }

        private void BindState(State state)
        {
            if (state != null)
            {
                nameText.Text = state.Name;
                synchronizedStateCheck.Checked = state.Synchronized;
                motionDiagram.Clear();
                motionDiagram.AddJoinMotions(state.Motions.ToArray());
                robot3DControl.JointMotions = motionDiagram.JointMotions;
                StateChanged = false;
            }
        }

        private void SaveCurrentState()
        {                  
            try
            {
                //if (stateList.SelectedIndex < 0) return;
                if (openStateIndex < 0) return;

                if (stateTabControl.SelectedTab == visualisationTabPage)
                {
                    motionDiagram.Clear();
                    motionDiagram.AddJoinMotions(robot3DControl.JointMotions.ToArray());                    
                }

                State changedState = new State(nameText.Text);
                changedState.Synchronized = synchronizedStateCheck.Checked;
                changedState.AddMotions(motionDiagram.JointMotions.ToArray());

                motion.States[openStateIndex] = changedState;
                StateChanged = false;

                UpdateStates();            
            }
            catch (Exception ex)
            {
                MessageBox.Show("Nastala neočakávaná chyba.\n\n" + ex.ToString(), "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }        

        private void nameText_TextChanged(object sender, EventArgs e)
        {
            StateChanged = true;
        }

        private void saveMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void openMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog.ShowDialog();
        }

        private void saveAsMenuItem_Click(object sender, EventArgs e)
        {
            SaveAs();
        }

        private void closeMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void newMenuItem_Click(object sender, EventArgs e)
        {
            
        }

        private void saveFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            motion.Save(saveFileDialog.FileName);
            openFileName = saveFileDialog.FileName;
        }

        private void openFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            LoadMotion(openFileDialog.FileName);
        }        

        private void actionPropertiesMenuItem_Click(object sender, EventArgs e)
        {
            MotionPropertiesForm frm = new MotionPropertiesForm();
            frm.MotionName = motion.Name;
            frm.MotionDescription = motion.Description;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                motion.Name = frm.MotionName;
                motion.Description = frm.MotionDescription;
            }
        }

        private void ConfigMenuItem_Click(object sender, EventArgs e)
        {
            ConfigForm frm = new ConfigForm();
            frm.ShowDialog();
        }

        private void previewButton_Click(object sender, EventArgs e)
        {
            try
            {
                SaveCurrentState();

                RunRobocupServer();

                RunRobotAgent();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Pri spúšťaní sa vyskytla neočakávaná chyba.\n\n" + ex.ToString(), null, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }        

        private void preview2Button_Click(object sender, EventArgs e)
        {
            try
            {                
                RunRobotAgent();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Pri spúšťaní sa vyskytla neočakávaná chyba.\n\n" + ex.ToString(), null, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void RunRobocupServer()
        {
            // Spusti server
            ProcessStartInfo info = new ProcessStartInfo();
            info.FileName = new System.IO.FileInfo(Program.Config.ServerFileName).FullName;
            info.WorkingDirectory =
                new System.IO.FileInfo(Program.Config.ServerFileName).Directory.FullName;            

            serverProcess = Process.Start(info);
            System.Threading.Thread.Sleep(3000);
        }

        private void RunRobotAgent()
        {
            RunRobotAgent(new RunSettings());
        }

        private void RunRobotAgent(RunSettings settings)
        {
            SaveCurrentState();

            // Vytvori docasny subor s pohybom
            Random rnd = new Random();
            string actionFile = Program.Config.TempFileName + rnd.Next() + ".tmp";

            RobotMotion newMotion = new RobotMotion(motion);
            newMotion.ApplySettings(settings);
            newMotion.Save(actionFile);            

            string absoluteFileName = new System.IO.FileInfo(Program.Config.AgentFileName).FullName;
            
            ProcessStartInfo processInfo =
                new ProcessStartInfo(absoluteFileName);
                        
            processInfo.Arguments = GetAgentArguments(actionFile);

            processInfo.WorkingDirectory =
                new System.IO.FileInfo(Program.Config.AgentFileName).Directory.FullName;

            // Spusti agenta
            agentProcess = Process.Start(processInfo);
        }

        private string GetAgentArguments(string actionFile)
        {
            string args = " ";

            args += "-file \"" + actionFile + "\" ";

            if (!string.IsNullOrEmpty(Program.Config.AgentServerIp))
                args += "-ip " + Program.Config.AgentServerIp + " ";
            if (Program.Config.AgentNoConsole)
                args += "-noconsole ";
            if (Program.Config.AgentNoDebug)
                args += "-nodebug ";

            args += Program.Config.AgentArguments;

            return args;
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            if (serverProcess != null)
            {
                try
                {

                    serverProcess.Kill();
                    agentProcess.Kill();
                    serverProcess = null;
                    agentProcess = null;
                }
                catch { }
            }
        }

        private void deleteMoveButton_Click(object sender, EventArgs e)
        {
            motionDiagram.DeleteSelected();            
            StateChanged = true;
        }

        private void editMoveButton_Click(object sender, EventArgs e)
        {
            EditMotion();
        }

        private void EditMotion()
        {
            if (motionDiagram.SelectedItem != null)
            {
                JointMotionEditor frm = new JointMotionEditor(motionDiagram.SelectedItem, 
                    motionDiagram.JointMotions.Except(
                    new List<JointMotion>() { (JointMotion)motionDiagram.SelectedItem }).ToList());
                frm.ShowDialog();

                if (frm.DialogResult == DialogResult.OK)
                {
                    motionDiagram.UpdateMotion(motionDiagram.SelectedIndex, frm.Result);
                    StateChanged = true;
                }
            }
        }

        
        private void synchronizedStateCheck_CheckedChanged(object sender, EventArgs e)
        {
            stateChanged = true;
        }

        private void deleteStateButton_Click(object sender, EventArgs e)
        {
            if (stateList.SelectedIndex >= 0)
            {
                if (openStateIndex == stateList.SelectedIndex) openStateIndex = -1;
                motion.States.RemoveAt(stateList.SelectedIndex);
                stateList.Items.RemoveAt(stateList.SelectedIndex);                
                stateList.SelectedIndex = -1;
            }
        }

        private void saveButton_Click(object sender, EventArgs e)
        {            
            SaveToLibrary();
        }

        private void SaveToLibrary()
        {
            SaveCurrentState();

            if (string.IsNullOrEmpty(openFileName))
            {
                MotionPropertiesForm frm = new MotionPropertiesForm();
                frm.ShowDialog(this);
                if (frm.DialogResult != DialogResult.OK) return;

                openFileName = Path.Combine(Path.Combine(Program.Config.LibraryPath, "Motions"),
                    frm.MotionName.Replace("\\", "-").Replace(":", "-").Replace("//", "-").Replace("*", "-") + ".rmo");
            }

            motion.Save(openFileName);
            Library.Library.Update();
        }


        private void SaveAs()
        {
            saveFileDialog.ShowDialog();
        }

        private void Save()
        {
            if (string.IsNullOrEmpty(openFileName))
            {
                SaveAs();
            }
            else
            {
                motion.Save(openFileName);
            }
        }

        private void motionDiagram_DoubleClick(object sender, EventArgs e)
        {
            EditMotion();
        }

        private void addMotionsVisuallyButton_Click(object sender, EventArgs e)
        {
            /*
            var window = new System.Windows.Window();
            window.Content = new RobotBehaviourEditor.MotionEditor.Visualisation.Robot3DControl(window)
                {
                    Width = window.Width,
                    Height = window.Height,
                };            
            window.ShowDialog();
            */

            
            RobotBehaviourEditor.MotionEditor.Visualisation.Robot3DForm frm = new RobotBehaviourEditor.MotionEditor.Visualisation.Robot3DForm();
            frm.ShowDialog(this);            
        }

        private void MotionEditorForm_Load(object sender, EventArgs e)
        {
            robot3DControl = new Robot3DControl();            
            elementHost1.HostContainer.Children.Add(robot3DControl);

            motionDiagram.VisualisationControl = robot3DControl;
        }

        private void stateTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (stateTabControl.SelectedTab == visualisationTabPage)
            {
                robot3DControl.JointMotions = motionDiagram.JointMotions;
                StateChanged = true;
            }
            else if (stateTabControl.SelectedTab == timelineTabPage)
            {
                motionDiagram.Clear();
                motionDiagram.AddJoinMotions(robot3DControl.JointMotions.ToArray());
                StateChanged = true;
            }
        }

        private void repeatButton_Click(object sender, EventArgs e)
        {
            RepetitionForm dlg = new RepetitionForm();
            dlg.ShowDialog();

            if (dlg.DialogResult == DialogResult.OK)
            {
                int delayAfterFirst = dlg.DelayAfterFirst;
                int delayBetweenRepetitions = dlg.DelayBetweenRepetitions;
                int repetitionCount = dlg.RepetitionCount;
                RepetitionType repetitionType = dlg.RepetitionType;

                if (repetitionType == RepetitionType.RepeatIndividually)
                    RepeatIndividually(delayAfterFirst, delayBetweenRepetitions, repetitionCount);
                else if (repetitionType == RepetitionType.RepeatAsGroup)
                    RepeatAsGroup(delayAfterFirst, delayBetweenRepetitions, repetitionCount);
            }
        }

        private void RepeatAsGroup(int delayAfterFirst, int delayBetweenRepetitions, int repetitionCount)
        {
            List<JointMotion> newMotions = new List<JointMotion>();
            int minTime = int.MaxValue;
            int maxTime = 0;
            foreach (int i in motionDiagram.SelectedIndices)
            {
                JointMotion motion = motionDiagram.JointMotions[i];
                if (motion.StartTime < minTime) minTime = motion.StartTime;
                if (motion.StartTime + motion.Duration > maxTime) maxTime = motion.StartTime + motion.Duration;
            }
            int interval = maxTime - minTime;

            int curTime = maxTime + delayAfterFirst;
            for (int j = 0; j < repetitionCount; j++)
            {                
                foreach (int i in motionDiagram.SelectedIndices)
                {
                    JointMotion motion = motionDiagram.JointMotions[i];
                    int time = motion.StartTime - minTime + curTime;
                    newMotions.Add(new JointMotion(motion) { StartTime = time });                    
                }
                curTime += interval + delayBetweenRepetitions;
            }
            motionDiagram.AddJoinMotions(newMotions.ToArray());
        }

        private void RepeatIndividually(int delayAfterFirst, int delayBetweenRepetitions, int repetitionCount)
        {
            List<JointMotion> newMotions = new List<JointMotion>();
            foreach (int i in motionDiagram.SelectedIndices)
            {
                JointMotion motion = motionDiagram.JointMotions[i];
                int curTime = motion.StartTime + motion.Duration + delayAfterFirst;

                for (int j = 0; j < repetitionCount; j++)
                {
                    newMotions.Add(new JointMotion(motion) { StartTime = curTime });
                    curTime += motion.Duration + delayBetweenRepetitions;
                }
            }
            motionDiagram.AddJoinMotions(newMotions.ToArray());
        }

        private void runSpeciallyButton_Click(object sender, EventArgs e)
        {
            RunSpeciallyForm frm = new RunSpeciallyForm(savedRunSettings);
            frm.ShowDialog(this);

            if (frm.DialogResult == DialogResult.OK)
            {
                savedRunSettings = frm.Settings;
                RunRobotAgent(frm.Settings);
            }
        }

        private void upButton_Click(object sender, EventArgs e)
        {
            motionDiagram.MoveUpSelected();
            motionDiagram.Invalidate();
        }

        private void downButton_Click(object sender, EventArgs e)
        {
            motionDiagram.MoveDownSelected();
            motionDiagram.Invalidate();
        }
        private void ExportButton_Click(object sender, EventArgs e)
        {
            ExportXML();
        }

        public static void ExportMovesToXml(string xmlFilePath, List<State> states, string xmlType)
        {
            try
            {

                List<JointMotion> motions4exp = new List<JointMotion>();

                List<State> newList = new List<State>(states.Count);

                states.ForEach((item) =>
                {
                    newList.Add(new State(item));
                });

                int count = 0;
                int lastEndTime = 0;
                List<JointMotion> temp2 = new List<JointMotion>();
                List<JointMotion> temp = new List<JointMotion>();
                for (int j = 0; j < newList.Count; j++)
                {////////////////////////////////////////////////////////////////////////////

                    Model.State state2exp = newList.ElementAt(j);

                    for (int i = 0; i < state2exp.Motions.Count; i++)
                    {


                        JointMotion mott = state2exp.Motions.ElementAt(i);
                        temp.Add(mott);

                        System.Console.Out.WriteLine("j -> " + j);
                        if (j > 0)
                        {
                            if (i == 0)
                            {
                                // sort pola obsahujuceho vsetky JointMtiony podla zaciatocneho casu
                                temp2.Sort(delegate(JointMotion jointM1, JointMotion jointM2)
                                {
                                    return jointM1.EndTime.CompareTo(jointM2.EndTime);
                                });
                                System.Console.Out.WriteLine("velkost temp2 -> " + temp2.Count);
                                System.Console.Out.WriteLine("[posledny v temp2 -> " + temp2.ElementAt(temp2.Count - 1).EndTime);
                                lastEndTime = temp2.ElementAt(temp2.Count - 1).EndTime;
                            }

                            System.Console.Out.WriteLine("posunut o -> " + lastEndTime);
                            for (int l = count; l < temp.Count; l++)
                            {
                                temp.ElementAt(l).StartTime += lastEndTime;
                                //temp.ElementAt(l).Duration += lastEndTime;
                                //temp.ElementAt(i).EndTime += temp.ElementAt(l).StartTime + temp.ElementAt(l).Duration;
                            }

                        }
                        temp2.Add(mott);
                        count++;

                    }


                }///////////////////////////////////////////////////////////////////////////////////

                for (int i = 0; i < temp.Count; i++)
                {//*********************************************************************



                    JointMotion mot = temp.ElementAt(i);
                    motions4exp.Add(mot);



                };//*********************************************************************

                XmlExporter exporter = new XmlExporter(xmlFilePath, motions4exp);
                if(xmlType.Equals(MotionEditorForm.XML_JIM_PLAYER))
                {
                    exporter.exportToXMLFileForJim();
                }
                else if (xmlType.Equals(MotionEditorForm.XML_CRITICAL_ERROR_PLAYER))
                {
                    exporter.exportToXMLFile();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Chyba pri exporte", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ExportXML()
        {

            Commands.ExportXmlMessageFile(motion.States);

        }

    }
}

