﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Shapes;
using System.Windows.Controls;
using System.Windows.Media.Media3D;
using System.Windows.Media;
using System.Xml.Serialization;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{
    public class Cuboid
    {

        #region Properties        
        Model3DGroup model3D = new Model3DGroup();
        [XmlIgnore]
        public Model3DGroup Model3D
        {
            get { return model3D; }            
        }
        
        Rect3D? originalRect3D = null;
        [XmlIgnore]
        private Rect3D? OriginalRect3D
        {
            get { return originalRect3D; }
            set { originalRect3D = value; }
        }

        private Rect3D rect3D = new Rect3D(0, 0, 0, 0, 0, 0);
        [XmlIgnore]
        public Rect3D Rect3D
        {
            get { return rect3D; }
            set
            {                    
                rect3D = value;

                if (model3D.Children.Count > 0)
                {
                    Model3D.Transform = new TranslateTransform3D(
                        originalRect3D.Value.X - rect3D.X,
                        originalRect3D.Value.Y - rect3D.Y,
                        originalRect3D.Value.Z - rect3D.Z);
                }
                else
                    originalRect3D = rect3D;
            }
        }

        [XmlAttribute("X")]
        public double X
        {
            get { return rect3D.X; }
            set { rect3D = new Rect3D(value, rect3D.Y, rect3D.Z, rect3D.SizeX, rect3D.SizeY, rect3D.SizeZ); }
        }

        [XmlAttribute("Y")]
        public double Y
        {
            get { return rect3D.Y; }
            set { rect3D = new Rect3D(rect3D.X, value, rect3D.Z, rect3D.SizeX, rect3D.SizeY, rect3D.SizeZ); }
        }

        [XmlAttribute("Z")]
        public double Z
        {
            get { return rect3D.Z; }
            set { rect3D = new Rect3D(rect3D.X, rect3D.Y, value, rect3D.SizeX, rect3D.SizeY, rect3D.SizeZ); }
        }

        [XmlAttribute("SizeX")]
        public double SizeX
        {
            get { return rect3D.SizeX; }
            set { rect3D = new Rect3D(rect3D.X, rect3D.Y, rect3D.Z, value, rect3D.SizeY, rect3D.SizeZ); }
        }

        [XmlAttribute("SizeY")]
        public double SizeY
        {
            get { return rect3D.SizeY; }
            set { rect3D = new Rect3D(rect3D.X, rect3D.Y, rect3D.Z, rect3D.SizeX, value, rect3D.SizeZ); }
        }

        [XmlAttribute("SizeZ")]
        public double SizeZ
        {
            get { return rect3D.SizeZ; }
            set { rect3D = new Rect3D(rect3D.X, rect3D.Y, rect3D.Z, rect3D.SizeX, rect3D.SizeY, value); }
        }
      
        private Brush materialBrush;
        [XmlIgnore()]
        public Brush MaterialBrush
        {
            get { return materialBrush; }
            set { materialBrush = value; }
        }

        #endregion

        public Cuboid()
            : this(new Rect3D(0, 0, 0, 0, 0, 0), new SolidColorBrush(Colors.DodgerBlue))
        {
        }

        public Cuboid(Rect3D rect3D)
            : this(rect3D, new SolidColorBrush(Colors.DodgerBlue))
        {
        }

        public Cuboid(Rect3D rect3D, Brush materialBrush)
        {            
            this.Rect3D = rect3D;
            this.MaterialBrush = materialBrush;

            Initialize();            
        }        

        private void Initialize()
        {
            Point3D[] cubePoints = new Point3D[]
            {
                new Point3D(rect3D.X, rect3D.Y, rect3D.Z),
                new Point3D(rect3D.X + rect3D.SizeX, rect3D.Y, rect3D.Z),
                new Point3D(rect3D.X + rect3D.SizeX, rect3D.Y, rect3D.Z + rect3D.SizeZ),
                new Point3D(rect3D.X, rect3D.Y, rect3D.Z + rect3D.SizeZ),
                new Point3D(rect3D.X, rect3D.Y + rect3D.SizeY, rect3D.Z),
                new Point3D(rect3D.X + rect3D.SizeX, rect3D.Y + rect3D.SizeY, rect3D.Z),
                new Point3D(rect3D.X + rect3D.SizeX, rect3D.Y + rect3D.SizeY, rect3D.Z + rect3D.SizeZ),
                new Point3D(rect3D.X, rect3D.Y + rect3D.SizeY, rect3D.Z + rect3D.SizeZ)
            };
            

            // Predna stena
            model3D.Children.Add(CreateRectangleModel(cubePoints[3], cubePoints[2], cubePoints[6], cubePoints[7]));

            // Prava stena
            model3D.Children.Add(CreateRectangleModel(cubePoints[2], cubePoints[1], cubePoints[5], cubePoints[6]));            

            // Zadna stena
            model3D.Children.Add(CreateRectangleModel(cubePoints[1], cubePoints[0], cubePoints[4], cubePoints[5]));            

            // Lava stena
            model3D.Children.Add(CreateRectangleModel(cubePoints[0], cubePoints[3], cubePoints[7], cubePoints[4]));            

            // Vrchna stena
            model3D.Children.Add(CreateRectangleModel(cubePoints[7], cubePoints[6], cubePoints[5], cubePoints[4]));            

            // Spodna stena
            model3D.Children.Add(CreateRectangleModel(cubePoints[2], cubePoints[3], cubePoints[0], cubePoints[1]));            
            
            //group.Children.Add(cube);            
        }

        public void InsertToViewport(Viewport3D viewport, string name, int id)
        {
            NamedModel3D model = new NamedModel3D();
            model.OriginalMaterial = new DiffuseMaterial(MaterialBrush);
            model.Content = model3D;
            model.Name = name;
            model.Id = id;

            viewport.Children.Add(model);
        }

        private Model3DGroup CreateRectangleModel(Point3D p0, Point3D p1, Point3D p2, Point3D p3)
        {
            Model3DGroup retval = new Model3DGroup();

            retval.Children.Add(CreateTriangleModel(p0, p1, p2,
                                new System.Windows.Point(0, 1),
                                new System.Windows.Point(1, 1),
                                new System.Windows.Point(1, 0)));
            retval.Children.Add(CreateTriangleModel(p0, p2, p3,
                                new System.Windows.Point(0, 1), 
                                new System.Windows.Point(1, 0), 
                                new System.Windows.Point(0, 0)));
            return retval;
        }

        private Model3DGroup CreateTriangleModel(Point3D p0, Point3D p1, Point3D p2, 
            System.Windows.Point textureC1, System.Windows.Point textureC2, System.Windows.Point textureC3)
        {
            MeshGeometry3D mesh = new MeshGeometry3D();
            mesh.Positions.Add(p0);
            mesh.Positions.Add(p1);
            mesh.Positions.Add(p2);
            mesh.TriangleIndices.Add(0);
            mesh.TriangleIndices.Add(1);
            mesh.TriangleIndices.Add(2);

            Vector3D normal = CalculateNormal(p0, p1, p2);
            mesh.Normals.Add(normal);
            mesh.Normals.Add(normal);
            mesh.Normals.Add(normal);

            mesh.TextureCoordinates.Add(textureC1);
            mesh.TextureCoordinates.Add(textureC2);
            mesh.TextureCoordinates.Add(textureC3);

            //Material material = new DiffuseMaterial(new SolidColorBrush(Colors.DodgerBlue));            
            Material material = new DiffuseMaterial(MaterialBrush);
            GeometryModel3D model = new GeometryModel3D(mesh, material);
            Model3DGroup group = new Model3DGroup();

            group.Children.Add(model);                        
            return group;
        }

        private Vector3D CalculateNormal(Point3D p0, Point3D p1, Point3D p2)
        {
            Vector3D v0 = new Vector3D(p1.X - p0.X, p1.Y - p0.Y, p1.Z - p0.Z);
            Vector3D v1 = new Vector3D(p2.X - p1.X, p2.Y - p1.Y, p2.Z - p1.Z);
            return Vector3D.CrossProduct(v0, v1);
        }

    }
}
