﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Media3D;
using System.Windows.Media;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{
    public class NamedModel3D : ModelVisual3D
    {
        private string name;
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        private int id;
        public int Id
        {
            get { return id; }
            set { id = value; }
        }

        private Material originalMaterial = null;
        public Material OriginalMaterial
        {
            get { return originalMaterial; }
            set { originalMaterial = value; }
        }

        private Material savedMaterial = null;
        private Material SavedMaterial
        {
            get { return savedMaterial; }
            set { savedMaterial = value; }
        }        

        public void ChangeMaterial(Material material)
        {                        
            ChangedMaterial(this.Content, material);  
        }

        private static void ChangedMaterial(Model3D model, Material material)
        {
            if (model is Model3DGroup)
            {
                foreach (var c in ((Model3DGroup)model).Children)
                {
                    ChangedMaterial(c, material);
                }
            }
            else if (model is GeometryModel3D)
            {
                ((GeometryModel3D)model).Material = material;
            }
        }

        private static Material GetMaterial(Model3D model)
        {
            if (model is Model3DGroup)
            {
                foreach (var c in ((Model3DGroup)model).Children)
                {
                    Material retval = GetMaterial(c);
                    if (retval != null) return retval;
                }
                return null;
            }
            else if (model is GeometryModel3D)
            {
                return ((GeometryModel3D)model).Material;
            }
            else
                return null;
        }

        public void SetOriginalMaterial()
        {
            ChangedMaterial(this.Content, originalMaterial);
        }

        internal void SaveMaterial()
        {
            savedMaterial = GetMaterial(this.Content);
        }

        internal void RestoreMaterial()
        {
            ChangedMaterial(this.Content, savedMaterial);
        }
    }
}
