﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.Windows.Controls;
using RobotBehaviourEditor.Model;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using RobotBehaviourEditor.Model.Physics;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{

    [XmlRoot("Robot3D")]
    public class Robot3D : IDisposable
    {
        #region Robot Parts (Properties)
        [XmlIgnore()]
        private Dictionary<int, Robot3DPart> parts = new Dictionary<int, Robot3DPart>(16);

        [XmlArray("Parts"), XmlArrayItem("Part", typeof(Robot3DPart))]
        public List<Robot3DPart> Parts
        {
            get { return parts.Select(x => x.Value).ToList(); }
            set 
            {
                parts.Clear();
                foreach (Robot3DPart p in value)
                {
                    parts.Add(p.Id, p);
                }
            }
        }

        private Robot3DPart head;
        //[XmlElement("Head")]
        [XmlIgnore()]
        public Robot3DPart Head
        {
            get { return head; }            
        }

        private Robot3DPart neck;
        //[XmlElement("Neck")]
        [XmlIgnore()]
        public Robot3DPart Neck
        {
            get { return neck; }            
        }

        private Robot3DPart torso;
        //[XmlElement("Torso")]
        [XmlIgnore()]
        public Robot3DPart Torso
        {
            get { return torso; }       
        }

        private Robot3DPart leftShoulder;
        //[XmlElement("LeftShoulder")]
        [XmlIgnore()]
        public Robot3DPart LeftShoulder
        {
            get { return leftShoulder; }
        }

        private Robot3DPart rightShoulder;
        //[XmlElement("RightShoulder")]
        [XmlIgnore()]
        public Robot3DPart RightShoulder
        {
            get { return rightShoulder; }
        }

        private Robot3DPart leftUpperArm;
        //[XmlElement("LeftUpperArm")]
        [XmlIgnore()]
        public Robot3DPart LeftUpperArm
        {
            get { return leftUpperArm; }            
        }

        private Robot3DPart rightUpperArm;
        //[XmlElement("RightUpperArm")]
        [XmlIgnore()]
        public Robot3DPart RightUpperArm
        {
            get { return rightUpperArm; }            
        }

        private Robot3DPart leftElbow;
        //[XmlElement("LeftElbow")]
        [XmlIgnore()]
        public Robot3DPart LeftElbow
        {
            get { return leftElbow; }
        }

        private Robot3DPart rightElbow;
        //[XmlElement("RightElbow")]
        [XmlIgnore()]
        public Robot3DPart RightElbow
        {
            get { return rightElbow; }
        }

        private Robot3DPart leftLowerArm;
        //[XmlElement("LeftLowerArm")]
        [XmlIgnore()]
        public Robot3DPart LeftLowerArm
        {
            get { return leftLowerArm; }            
        }

        private Robot3DPart rightLowerArm;
        //[XmlElement("RightLowerArm")]
        [XmlIgnore()]
        public Robot3DPart RightLowerArm
        {
            get { return rightLowerArm; }            
        }

        private Robot3DPart leftHip1;
        //[XmlElement("LeftHip1")]
        [XmlIgnore()]
        public Robot3DPart LeftHip1
        {
            get { return leftHip1; } 
        }

        private Robot3DPart rightHip1;
        //[XmlElement("RightHip1")]
        [XmlIgnore()]
        public Robot3DPart RightHip1
        {
            get { return rightHip1; }
        }

        private Robot3DPart leftHip2;
        //[XmlElement("LeftHip2")]
        [XmlIgnore()]
        public Robot3DPart LeftHip2
        {
            get { return leftHip2; }
        }

        private Robot3DPart rightHip2;
        //[XmlElement("RightHip2")]
        [XmlIgnore()]
        public Robot3DPart RightHip2
        {
            get { return rightHip2; }
        }

        private Robot3DPart leftThigh;
        //[XmlElement("LeftThigh")]
        [XmlIgnore()]
        public Robot3DPart LeftThigh
        {
            get { return leftThigh; }
        }

        private Robot3DPart rightThigh;
        //[XmlElement("RightThigh")]
        [XmlIgnore()]
        public Robot3DPart RightThigh
        {
            get { return rightThigh; }
        }

        private Robot3DPart leftShank;
        //[XmlElement("LeftShank")]
        [XmlIgnore()]
        public Robot3DPart LeftShank
        {
            get { return leftShank; }            
        }

        private Robot3DPart rightShank;
        //[XmlElement("LeftShank")]
        [XmlIgnore()]
        public Robot3DPart RightShank
        {
            get { return rightShank; }            
        }

        private Robot3DPart leftAnkle;
        //[XmlElement("LeftAnkle")]
        [XmlIgnore()]
        public Robot3DPart LeftAnkle
        {
            get { return leftAnkle; }
        }

        private Robot3DPart rightAnkle;
        //[XmlElement("RightAnkle")]
        [XmlIgnore()]
        public Robot3DPart RightAnkle
        {
            get { return rightAnkle; }
        }

        private Robot3DPart leftFoot;
        //[XmlElement("LeftFoot")]
        [XmlIgnore()]
        public Robot3DPart LeftFoot
        {
            get { return leftFoot; }        
        }

        private Robot3DPart rightFoot;
        //[XmlElement("RightFoot")]
        [XmlIgnore()]
        public Robot3DPart RightFoot
        {
            get { return rightFoot; }
        }
        
        Cuboid centerOfMassCuboid = null;
        [XmlIgnore()]
        public Cuboid CenterOfMassCuboid
        {
            get { return centerOfMassCuboid; }
            set { centerOfMassCuboid = value; }
        }

        #endregion

        public double DefaultScale
        {
            get;
            set;
        }

        public Point3D DefaultCameraPosition
        {
            get;
            set;
        }

        private Robot3D()
        {             
        }

        public static Robot3D GetRobot()
        {
            if (Program.Config.Visualisation3dModel == "nao")
            {
                return Robot3D.GetNaoRobot();
            }
            else if (Program.Config.Visualisation3dModel!=null && Program.Config.Visualisation3dModel.EndsWith(".r3d"))
            {
                return Robot3D.GetRobotFromFile(
                    Path.Combine(
                        Program.Config.Models3dPath, 
                        Program.Config.Visualisation3dModel));
            }
            else
            {
                return Robot3D.GetDefaultRobot();
            }            
        }

        public static Robot3D GetDefaultRobot()
        {
            Robot3D r = new Robot3D();
            r.DefaultScale = 1;
            r.DefaultCameraPosition = new Point3D(71, 105, 50);

            // Hlava
            r.head = new Robot3DPart(r, "Hlava", (int)JointTypes.Head);
            r.head.Code = "HE2";
            r.head.Cuboid = new Cuboid(new Rect3D(5, 50, -1.5, 10, 8, 5), new ImageBrush(Images.FaceTexture));
            r.head.RotationAxis = new Vector3D(-1, 0, 0);
            r.head.RotationPoint = new Point3D(r.head.Cuboid.Rect3D.X + r.head.Cuboid.Rect3D.SizeX / 2, r.head.Cuboid.Rect3D.Y, r.head.Cuboid.Rect3D.Z + r.head.Cuboid.Rect3D.SizeZ / 2);
            r.head.MinimumAngle = -45; r.head.MaximumAngle = 45;

            // Krk
            r.neck = new Robot3DPart(r, "Krk", (int)JointTypes.Neck);
            r.neck.Code = "HE1";
            r.neck.Cuboid = new Cuboid(new Rect3D(8, 48, 0, 4, 2, 2), new ImageBrush(Images.MetalTexture));
            r.neck.RotationAxis = new Vector3D(0, -1, 0);
            r.neck.RotationPoint = new Point3D(r.neck.Cuboid.Rect3D.X + r.neck.Cuboid.Rect3D.SizeX / 2, r.neck.Cuboid.Rect3D.Y, r.neck.Cuboid.Rect3D.Z + r.neck.Cuboid.Rect3D.SizeZ / 2);
            r.neck.MinimumAngle = -120; r.neck.MaximumAngle = 120;

            // Trup
            r.torso = new Robot3DPart(r, "Trup", (int)JointTypes.Torso);
            r.torso.Code = "T";
            r.torso.Cuboid = new Cuboid(new Rect3D(4, 32, 0, 12, 16, 4), new ImageBrush(Images.MetalTexture));
            r.torso.RotationPoint = new Point3D(r.torso.Cuboid.Rect3D.X, r.torso.Cuboid.Rect3D.Y, r.torso.Cuboid.Rect3D.Z + r.torso.Cuboid.Rect3D.SizeZ / 2);

            // Ruky
            // Plecia
            r.rightShoulder = new Robot3DPart(r, "Pravé plece", (int)JointTypes.RightShoulder);
            r.rightShoulder.Code = "RAE1";
            r.rightShoulder.Cuboid = new Cuboid(new Rect3D(0, 45, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.rightShoulder.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightShoulder.MinimumAngle = -120; r.rightShoulder.MaximumAngle = 120;

            r.leftShoulder = new Robot3DPart(r, "Ľavé plece", (int)JointTypes.LeftShoulder);
            r.leftShoulder.Code = "LAE1";
            r.leftShoulder.Cuboid = new Cuboid(new Rect3D(16, 45, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.leftShoulder.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftShoulder.MinimumAngle = -120; r.leftShoulder.MaximumAngle = 120;

            // Ramena
            r.rightUpperArm = new Robot3DPart(r, "Pravé rameno", (int)JointTypes.RightUpperArm);
            r.rightUpperArm.Code = "RAE2";
            r.rightUpperArm.Cuboid = new Cuboid(new Rect3D(0, 45, 0, 4, 3, 12), new ImageBrush(Images.MetalTexture));
            r.rightUpperArm.RotationAxis = new Vector3D(0, 1, 0);
            r.rightUpperArm.RotationPoint = new Point3D(r.rightUpperArm.Cuboid.Rect3D.X + r.rightUpperArm.Cuboid.Rect3D.SizeX,
                r.rightUpperArm.Cuboid.Rect3D.Y + r.rightUpperArm.Cuboid.Rect3D.SizeY / 2, r.rightUpperArm.Cuboid.Rect3D.Z + 1.5);
            r.rightUpperArm.MinimumAngle = -95; r.rightUpperArm.MaximumAngle = 1;
            r.rightUpperArm.AddRelatedPart(r.rightShoulder);

            r.leftUpperArm = new Robot3DPart(r, "Ľavé rameno", (int)JointTypes.LeftUpperArm);
            r.leftUpperArm.Code = "LAE2";
            r.leftUpperArm.Cuboid = new Cuboid(new Rect3D(16, 45, 0, 4, 3, 12), new ImageBrush(Images.MetalTexture));
            r.leftUpperArm.RotationAxis = new Vector3D(0, 1, 0);
            r.leftUpperArm.RotationPoint = new Point3D(r.leftUpperArm.Cuboid.Rect3D.X,
                r.leftUpperArm.Cuboid.Rect3D.Y + r.leftUpperArm.Cuboid.Rect3D.SizeY / 2, r.leftUpperArm.Cuboid.Rect3D.Z + 1.5);
            r.leftUpperArm.MinimumAngle = -1; r.leftUpperArm.MaximumAngle = 95;
            r.leftUpperArm.AddRelatedPart(r.leftShoulder);

            r.rightShoulder.RotationPoint = r.rightUpperArm.RotationPoint;
            r.leftShoulder.RotationPoint = r.leftUpperArm.RotationPoint;

            // Lakte
            r.rightElbow = new Robot3DPart(r, "Pravý lakeť", (int)JointTypes.RightElbow);
            r.rightElbow.Code = "RAE3";
            r.rightElbow.Cuboid = new Cuboid(new Rect3D(0.1, 45, 12, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.rightElbow.RotationAxis = new Vector3D(0, 0, 1);
            r.rightElbow.MinimumAngle = -120; r.rightElbow.MaximumAngle = 120;

            r.leftElbow = new Robot3DPart(r, "Ľavý lakeť", (int)JointTypes.LeftElbow);
            r.leftElbow.Code = "LAE3";
            r.leftElbow.Cuboid = new Cuboid(new Rect3D(16.1, 45, 12, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.leftElbow.RotationAxis = new Vector3D(0, 0, 1);
            r.leftElbow.MinimumAngle = -120; r.leftElbow.MaximumAngle = 120;

            // Predlaktia
            r.rightLowerArm = new Robot3DPart(r, "Pravé predlaktie", (int)JointTypes.RightLowerArm);
            r.rightLowerArm.Code = "RAE4";
            r.rightLowerArm.Cuboid = new Cuboid(new Rect3D(0.1, 45, 12, 4, 3, 12), new ImageBrush(Images.MetalTexture));
            r.rightLowerArm.RotationAxis = new Vector3D(0, 1, 0);
            r.rightLowerArm.RotationPoint = new Point3D(r.rightLowerArm.Cuboid.Rect3D.X + r.rightLowerArm.Cuboid.Rect3D.SizeX / 2,
                r.rightLowerArm.Cuboid.Rect3D.Y + r.rightLowerArm.Cuboid.Rect3D.SizeY / 2, r.rightLowerArm.Cuboid.Rect3D.Z);
            r.rightLowerArm.MinimumAngle = -1; r.rightLowerArm.MaximumAngle = 90;
            r.rightLowerArm.AddRelatedPart(r.rightElbow);

            r.leftLowerArm = new Robot3DPart(r, "Ľavé predlaktie", (int)JointTypes.LeftLowerArm);
            r.leftLowerArm.Code = "LAE4";
            r.leftLowerArm.Cuboid = new Cuboid(new Rect3D(16.1, 45, 12, 4, 3, 12), new ImageBrush(Images.MetalTexture));
            r.leftLowerArm.RotationAxis = new Vector3D(0, 1, 0);
            r.leftLowerArm.RotationPoint = new Point3D(r.leftLowerArm.Cuboid.Rect3D.X + r.leftLowerArm.Cuboid.Rect3D.SizeX / 2,
                r.leftLowerArm.Cuboid.Rect3D.Y + r.leftLowerArm.Cuboid.Rect3D.SizeY / 2, r.leftLowerArm.Cuboid.Rect3D.Z);
            r.leftLowerArm.MinimumAngle = -90; r.leftLowerArm.MaximumAngle = 1;
            r.leftLowerArm.AddRelatedPart(r.leftElbow);

            r.rightElbow.RotationPoint = r.rightLowerArm.RotationPoint;
            r.leftElbow.RotationPoint = r.leftLowerArm.RotationPoint;

            // Boky
            r.rightHip1 = new Robot3DPart(r, "Pravý bok 1", (int)JointTypes.RightHip1);
            r.rightHip1.Code = "RLE1";
            r.rightHip1.Cuboid = new Cuboid(new Rect3D(4, 17, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.rightHip1.RotationAxis = new Vector3D(0.7071, 0.7071, 0);
            r.rightHip1.MinimumAngle = -90; r.rightHip1.MaximumAngle = 1;

            r.leftHip1 = new Robot3DPart(r, "Ľavý bok 1", (int)JointTypes.LeftHip1);
            r.leftHip1.Code = "LLE1";
            r.leftHip1.Cuboid = new Cuboid(new Rect3D(10, 17, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.leftHip1.RotationAxis = new Vector3D(0.7071, -0.7071, 0);
            r.leftHip1.MinimumAngle = -90; r.leftHip1.MaximumAngle = 1;

            r.rightHip2 = new Robot3DPart(r, "Pravý bok 2", (int)JointTypes.RightHip2);
            r.rightHip2.Code = "RLE2";
            r.rightHip2.Cuboid = new Cuboid(new Rect3D(4, 17, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.rightHip2.RotationAxis = new Vector3D(0, 0, 1);
            r.rightHip2.MinimumAngle = -45; r.rightHip2.MaximumAngle = 25;

            r.leftHip2 = new Robot3DPart(r, "Ľavý bok 2", (int)JointTypes.LeftHip2);
            r.leftHip2.Code = "LLE2";
            r.leftHip2.Cuboid = new Cuboid(new Rect3D(10, 17, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.leftHip2.RotationAxis = new Vector3D(0, 0, 1);
            r.leftHip2.MinimumAngle = -25; r.leftHip2.MaximumAngle = 45;

            // Nohy
            // Stehna
            r.rightThigh = new Robot3DPart(r, "Pravé stehno", (int)JointTypes.RightThigh);
            r.rightThigh.Code = "RLE3";
            r.rightThigh.Cuboid = new Cuboid(new Rect3D(4, 17, 0, 6, 15, 3), new ImageBrush(Images.MetalTexture));
            r.rightThigh.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightThigh.RotationPoint = new Point3D(r.rightThigh.Cuboid.Rect3D.X + r.rightThigh.Cuboid.Rect3D.SizeX / 2,
                r.rightThigh.Cuboid.Rect3D.Y + r.rightThigh.Cuboid.Rect3D.SizeY, r.rightThigh.Cuboid.Rect3D.Z + r.rightThigh.Cuboid.Rect3D.SizeZ / 2);
            r.rightThigh.MinimumAngle = -25; r.rightThigh.MaximumAngle = 100;
            r.rightThigh.AddRelatedPart(r.rightHip1);
            r.rightThigh.AddRelatedPart(r.rightHip2);

            r.leftThigh = new Robot3DPart(r, "Ľavé stehno", (int)JointTypes.LeftThigh);
            r.leftThigh.Code = "LLE3";
            r.leftThigh.Cuboid = new Cuboid(new Rect3D(10, 17, 0, 6, 15, 3), new ImageBrush(Images.MetalTexture));
            r.leftThigh.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftThigh.RotationPoint = new Point3D(r.leftThigh.Cuboid.Rect3D.X + r.leftThigh.Cuboid.Rect3D.SizeX / 2,
                r.leftThigh.Cuboid.Rect3D.Y + r.leftThigh.Cuboid.Rect3D.SizeY, r.leftThigh.Cuboid.Rect3D.Z + r.leftThigh.Cuboid.Rect3D.SizeZ / 2);
            r.leftThigh.MinimumAngle = -25; r.leftThigh.MaximumAngle = 100;
            r.leftThigh.AddRelatedPart(r.leftHip1);
            r.leftThigh.AddRelatedPart(r.leftHip2);

            r.rightHip1.RotationPoint = r.rightThigh.RotationPoint;
            r.leftHip1.RotationPoint = r.leftThigh.RotationPoint;
            r.rightHip2.RotationPoint = r.rightThigh.RotationPoint;
            r.leftHip2.RotationPoint = r.leftThigh.RotationPoint;

            // Lytka
            r.rightShank = new Robot3DPart(r, "Pravé lýtko", (int)JointTypes.RightShank);
            r.rightShank.Code = "RLE4";
            r.rightShank.Cuboid = new Cuboid(new Rect3D(4.1, 2, 0, 6, 15, 3), new ImageBrush(Images.MetalTexture));
            r.rightShank.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightShank.RotationPoint = new Point3D(r.rightShank.Cuboid.Rect3D.X,
                r.rightShank.Cuboid.Rect3D.Y + r.rightShank.Cuboid.Rect3D.SizeY, r.rightShank.Cuboid.Rect3D.Z + r.rightShank.Cuboid.Rect3D.SizeZ / 2);
            r.rightShank.MinimumAngle = -130; r.rightShank.MaximumAngle = 1;

            r.leftShank = new Robot3DPart(r, "Ľavé lýtko", (int)JointTypes.LeftShank);
            r.leftShank.Code = "LLE4";
            r.leftShank.Cuboid = new Cuboid(new Rect3D(10.1, 2, 0, 6, 15, 3), new ImageBrush(Images.MetalTexture));
            r.leftShank.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftShank.RotationPoint = new Point3D(r.leftShank.Cuboid.Rect3D.X,
                r.leftShank.Cuboid.Rect3D.Y + r.leftShank.Cuboid.Rect3D.SizeY, r.leftShank.Cuboid.Rect3D.Z + r.leftShank.Cuboid.Rect3D.SizeZ / 2);
            r.leftShank.MinimumAngle = -130; r.leftShank.MaximumAngle = 1;

            // Členky
            r.rightAnkle = new Robot3DPart(r, "Pravý členok", (int)JointTypes.RightAnkle);
            r.rightAnkle.Code = "RLE5";
            r.rightAnkle.Cuboid = new Cuboid(new Rect3D(4, 0, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.rightAnkle.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightAnkle.MinimumAngle = -45; r.rightAnkle.MaximumAngle = 75;

            r.leftAnkle = new Robot3DPart(r, "Ľavý členok", (int)JointTypes.LeftAnkle);
            r.leftAnkle.Code = "LLE5";
            r.leftAnkle.Cuboid = new Cuboid(new Rect3D(10, 0, 0, 0.1, 0.1, 0.1), new ImageBrush(Images.MetalTexture));
            r.leftAnkle.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftAnkle.MinimumAngle = -45; r.leftAnkle.MaximumAngle = 75;


            // Chodidla
            r.rightFoot = new Robot3DPart(r, "Pravé chodidlo", (int)JointTypes.RightFoot);
            r.rightFoot.Code = "RLE6";
            r.rightFoot.Cuboid = new Cuboid(new Rect3D(4, 0, 0, 6, 2, 6), new ImageBrush(Images.MetalTexture));
            r.rightFoot.RotationAxis = new Vector3D(0, 0, 1);
            r.rightFoot.RotationPoint = new Point3D(r.rightFoot.Cuboid.Rect3D.X + r.rightFoot.Cuboid.Rect3D.SizeX / 2,
                r.rightFoot.Cuboid.Rect3D.Y + r.rightFoot.Cuboid.Rect3D.SizeY, r.rightFoot.Cuboid.Rect3D.Z);
            r.rightFoot.MinimumAngle = -25; r.rightFoot.MaximumAngle = 45;
            r.rightFoot.AddRelatedPart(r.rightAnkle);

            r.leftFoot = new Robot3DPart(r, "Ľavé chodidlo", (int)JointTypes.LeftFoot);
            r.leftFoot.Code = "LLE6";
            r.leftFoot.Cuboid = new Cuboid(new Rect3D(10, 0, 0, 6, 2, 6), new ImageBrush(Images.MetalTexture));
            r.leftFoot.RotationAxis = new Vector3D(0, 0, 1);
            r.leftFoot.RotationPoint = new Point3D(r.leftFoot.Cuboid.Rect3D.X + r.leftFoot.Cuboid.Rect3D.SizeX / 2,
                r.leftFoot.Cuboid.Rect3D.Y + r.leftFoot.Cuboid.Rect3D.SizeY, r.leftFoot.Cuboid.Rect3D.Z);
            r.leftFoot.MinimumAngle = -45; r.leftFoot.MaximumAngle = 25;
            r.leftFoot.AddRelatedPart(r.leftAnkle);

            r.rightAnkle.RotationPoint = r.rightFoot.RotationPoint;
            r.leftAnkle.RotationPoint = r.leftFoot.RotationPoint;

            // Spoje
            r.head.JoinWith(r.neck);
            r.neck.JoinWith(r.torso);
            r.rightShoulder.JoinWith(r.torso); r.leftShoulder.JoinWith(r.torso);
            r.rightUpperArm.JoinWith(r.rightShoulder); r.leftUpperArm.JoinWith(r.leftShoulder);
            r.rightElbow.JoinWith(r.rightUpperArm); r.leftElbow.JoinWith(r.leftUpperArm);
            r.rightLowerArm.JoinWith(r.rightElbow); r.leftLowerArm.JoinWith(r.leftElbow);
            r.rightHip1.JoinWith(r.torso); r.leftHip1.JoinWith(r.torso);
            r.rightHip2.JoinWith(r.rightHip1); r.leftHip2.JoinWith(r.leftHip1);
            r.rightThigh.JoinWith(r.rightHip2); r.leftThigh.JoinWith(r.leftHip2);
            r.rightShank.JoinWith(r.rightThigh); r.leftShank.JoinWith(r.leftThigh);
            r.rightAnkle.JoinWith(r.rightShank); r.leftAnkle.JoinWith(r.leftShank);
            r.rightFoot.JoinWith(r.rightAnkle); r.leftFoot.JoinWith(r.leftAnkle);

            return r;
        }

        public static Robot3D GetNaoRobot()
        {            
            Robot3D r = new Robot3D();
            r.DefaultScale = 0.008312457381923084;
            r.DefaultCameraPosition = new Point3D(71, 50, 50);

            // Hlava
            r.head = new Robot3DPart(r, "Hlava", (int)JointTypes.Head);
            r.head.Code = "HE2";
            r.head.Cuboid = new Cuboid(new Rect3D(-0.065, 0.09, -0.065, 0.1300, 0.1300, 0.1300), new ImageBrush(Images.FaceTexture));
            r.head.RotationAxis = new Vector3D(-1, 0, 0);
            r.head.RotationPoint = new Point3D(r.head.Cuboid.Rect3D.X + r.head.Cuboid.Rect3D.SizeX / 2, r.head.Cuboid.Rect3D.Y, r.head.Cuboid.Rect3D.Z + r.head.Cuboid.Rect3D.SizeZ / 2);
            r.head.MinimumAngle = -45; r.head.MaximumAngle = 45;

            // Krk
            r.neck = new Robot3DPart(r, "Krk", (int)JointTypes.Neck);
            r.neck.Code = "HE1";
            r.neck.Cuboid = new Cuboid(new Rect3D(-0.015, 0.05, -0.015, 0.0300, 0.0800, 0.0300), new ImageBrush(Images.MetalTexture));
            r.neck.RotationAxis = new Vector3D(0, -1, 0);
            r.neck.RotationPoint = new Point3D(r.neck.Cuboid.Rect3D.X + r.neck.Cuboid.Rect3D.SizeX / 2, r.neck.Cuboid.Rect3D.Y, r.neck.Cuboid.Rect3D.Z + r.neck.Cuboid.Rect3D.SizeZ / 2);
            r.neck.MinimumAngle = -120; r.neck.MaximumAngle = 120;

            // Trup
            r.torso = new Robot3DPart(r, "Trup", (int)JointTypes.Torso);
            r.torso.Code = "T";
            r.torso.Cuboid = new Cuboid(new Rect3D(-0.05, -0.09, -0.05, 0.1000, 0.1800, 0.1000), new ImageBrush(Images.MetalTexture));
            r.torso.RotationPoint = new Point3D(r.torso.Cuboid.Rect3D.X, r.torso.Cuboid.Rect3D.Y, r.torso.Cuboid.Rect3D.Z + r.torso.Cuboid.Rect3D.SizeZ / 2);

            // Ruky
            // Plecia
            r.rightShoulder = new Robot3DPart(r, "Pravé plece", (int)JointTypes.RightShoulder);
            r.rightShoulder.Code = "RAE1";
            r.rightShoulder.Cuboid = new Cuboid(new Rect3D(-0.108, 0.065, -0.01, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            r.rightShoulder.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightShoulder.MinimumAngle = -120; r.rightShoulder.MaximumAngle = 120;

            r.leftShoulder = new Robot3DPart(r, "Ľavé plece", (int)JointTypes.LeftShoulder);
            r.leftShoulder.Code = "LAE1";
            r.leftShoulder.Cuboid = new Cuboid(new Rect3D(0.088, 0.065, -0.01, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            r.leftShoulder.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftShoulder.MinimumAngle = -120; r.leftShoulder.MaximumAngle = 120;

            // Ramena
            r.rightUpperArm = new Robot3DPart(r, "Pravé rameno", (int)JointTypes.RightUpperArm);
            r.rightUpperArm.Code = "RAE2";
            r.rightUpperArm.Cuboid = new Cuboid(new Rect3D(-0.143, 0.045, -0.02, 0.0700, 0.0600, 0.0800), new ImageBrush(Images.MetalTexture));
            r.rightUpperArm.RotationAxis = new Vector3D(0, 1, 0);
            r.rightUpperArm.RotationPoint = new Point3D(r.rightUpperArm.Cuboid.Rect3D.X + r.rightUpperArm.Cuboid.Rect3D.SizeX,
                r.rightUpperArm.Cuboid.Rect3D.Y + r.rightUpperArm.Cuboid.Rect3D.SizeY / 2, r.rightUpperArm.Cuboid.Rect3D.Z + r.rightUpperArm.Cuboid.Rect3D.SizeY / 2);
            r.rightUpperArm.MinimumAngle = -95; r.rightUpperArm.MaximumAngle = 1;
            r.rightUpperArm.AddRelatedPart(r.rightShoulder);

            r.leftUpperArm = new Robot3DPart(r, "Ľavé rameno", (int)JointTypes.LeftUpperArm);
            r.leftUpperArm.Code = "LAE2";
            r.leftUpperArm.Cuboid = new Cuboid(new Rect3D(0.073, 0.045, -0.02, 0.0700, 0.0600, 0.0800), new ImageBrush(Images.MetalTexture));
            r.leftUpperArm.RotationAxis = new Vector3D(0, 1, 0);
            r.leftUpperArm.RotationPoint = new Point3D(r.leftUpperArm.Cuboid.Rect3D.X,
                r.leftUpperArm.Cuboid.Rect3D.Y + r.leftUpperArm.Cuboid.Rect3D.SizeY / 2, r.leftUpperArm.Cuboid.Rect3D.Z + r.rightUpperArm.Cuboid.Rect3D.SizeY / 2);
            r.leftUpperArm.MinimumAngle = -1; r.leftUpperArm.MaximumAngle = 95;
            r.leftUpperArm.AddRelatedPart(r.leftShoulder);

            r.rightShoulder.RotationPoint = r.rightUpperArm.RotationPoint;
            r.leftShoulder.RotationPoint = r.leftUpperArm.RotationPoint;

            // Lakte
            r.rightElbow = new Robot3DPart(r, "Pravý lakeť", (int)JointTypes.RightElbow);
            r.rightElbow.Code = "RAE3";
            r.rightElbow.Cuboid = new Cuboid(new Rect3D(-0.108, 0.074, 0.08, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            r.rightElbow.RotationAxis = new Vector3D(0, 0, 1);
            r.rightElbow.MinimumAngle = -120; r.rightElbow.MaximumAngle = 120;

            r.leftElbow = new Robot3DPart(r, "Ľavý lakeť", (int)JointTypes.LeftElbow);
            r.leftElbow.Code = "LAE3";
            r.leftElbow.Cuboid = new Cuboid(new Rect3D(0.088, 0.074, 0.08, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            r.leftElbow.RotationAxis = new Vector3D(0, 0, 1);
            r.leftElbow.MinimumAngle = -120; r.leftElbow.MaximumAngle = 120;

            // Predlaktia
            r.rightLowerArm = new Robot3DPart(r, "Pravé predlaktie", (int)JointTypes.RightLowerArm);
            r.rightLowerArm.Code = "RAE4";
            r.rightLowerArm.Cuboid = new Cuboid(new Rect3D(-0.123, 0.059, 0.085, 0.0500, 0.0500, 0.1100), new ImageBrush(Images.MetalTexture));
            r.rightLowerArm.RotationAxis = new Vector3D(0, 1, 0);
            r.rightLowerArm.RotationPoint = new Point3D(r.rightLowerArm.Cuboid.Rect3D.X + r.rightLowerArm.Cuboid.Rect3D.SizeX / 2,
                r.rightLowerArm.Cuboid.Rect3D.Y + r.rightLowerArm.Cuboid.Rect3D.SizeY / 2, r.rightLowerArm.Cuboid.Rect3D.Z);
            r.rightLowerArm.MinimumAngle = -1; r.rightLowerArm.MaximumAngle = 90;
            r.rightLowerArm.AddRelatedPart(r.rightElbow);

            r.leftLowerArm = new Robot3DPart(r, "Ľavé predlaktie", (int)JointTypes.LeftLowerArm);
            r.leftLowerArm.Code = "LAE4";
            r.leftLowerArm.Cuboid = new Cuboid(new Rect3D(0.073, 0.059, 0.085, 0.0500, 0.0500, 0.1100), new ImageBrush(Images.MetalTexture)); 
            r.leftLowerArm.RotationAxis = new Vector3D(0, 1, 0);
            r.leftLowerArm.RotationPoint = new Point3D(r.leftLowerArm.Cuboid.Rect3D.X + r.leftLowerArm.Cuboid.Rect3D.SizeX / 2,
                r.leftLowerArm.Cuboid.Rect3D.Y + r.leftLowerArm.Cuboid.Rect3D.SizeY / 2, r.leftLowerArm.Cuboid.Rect3D.Z);
            r.leftLowerArm.MinimumAngle = -90; r.leftLowerArm.MaximumAngle = 1;
            r.leftLowerArm.AddRelatedPart(r.leftElbow);

            r.rightElbow.RotationPoint = r.rightLowerArm.RotationPoint;
            r.leftElbow.RotationPoint = r.leftLowerArm.RotationPoint;

            // Boky
            r.rightHip1 = new Robot3DPart(r, "Pravý bok 1", (int)JointTypes.RightHip1);
            r.rightHip1.Code = "RLE1";
            r.rightHip1.Cuboid = new Cuboid(new Rect3D(-0.065, -0.125, -0.02, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            r.rightHip1.RotationAxis = new Vector3D(0.7071, 0.7071, 0);
            r.rightHip1.MinimumAngle = -90; r.rightHip1.MaximumAngle = 1;

            r.leftHip1 = new Robot3DPart(r, "Ľavý bok 1", (int)JointTypes.LeftHip1);
            r.leftHip1.Code = "LLE1";
            r.leftHip1.Cuboid = new Cuboid(new Rect3D(0.045, -0.125, -0.02, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            r.leftHip1.RotationAxis = new Vector3D(0.7071, -0.7071, 0);
            r.leftHip1.MinimumAngle = -90; r.leftHip1.MaximumAngle = 1;

            r.rightHip2 = new Robot3DPart(r, "Pravý bok 2", (int)JointTypes.RightHip2);
            r.rightHip2.Code = "RLE2";
            r.rightHip2.Cuboid = new Cuboid(new Rect3D(-0.065, -0.125, -0.02, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            r.rightHip2.RotationAxis = new Vector3D(0, 0, 1);
            r.rightHip2.MinimumAngle = -45; r.rightHip2.MaximumAngle = 25;

            r.leftHip2 = new Robot3DPart(r, "Ľavý bok 2", (int)JointTypes.LeftHip2);
            r.leftHip2.Code = "LLE2";
            r.leftHip2.Cuboid = new Cuboid(new Rect3D(0.045, -0.125, -0.02, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            r.leftHip2.RotationAxis = new Vector3D(0, 0, 1);
            r.leftHip2.MinimumAngle = -25; r.leftHip2.MaximumAngle = 45;

            // Nohy
            // Stehna
            r.rightThigh = new Robot3DPart(r, "Pravé stehno", (int)JointTypes.RightThigh);
            r.rightThigh.Code = "RLE3";
            r.rightThigh.Cuboid = new Cuboid(new Rect3D(-0.09, -0.225, -0.035, 0.0700, 0.1400, 0.0700), new ImageBrush(Images.MetalTexture));
            r.rightThigh.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightThigh.RotationPoint = new Point3D(r.rightThigh.Cuboid.Rect3D.X + r.rightThigh.Cuboid.Rect3D.SizeX / 2,
                r.rightThigh.Cuboid.Rect3D.Y + r.rightThigh.Cuboid.Rect3D.SizeY, r.rightThigh.Cuboid.Rect3D.Z + r.rightThigh.Cuboid.Rect3D.SizeZ / 2);
            r.rightThigh.MinimumAngle = -25; r.rightThigh.MaximumAngle = 100;
            r.rightThigh.AddRelatedPart(r.rightHip1);
            r.rightThigh.AddRelatedPart(r.rightHip2);

            r.leftThigh = new Robot3DPart(r, "Ľavé stehno", (int)JointTypes.LeftThigh);
            r.leftThigh.Code = "LLE3";
            r.leftThigh.Cuboid = new Cuboid(new Rect3D(0.02, -0.225, -0.035, 0.0700, 0.1400, 0.0700), new ImageBrush(Images.MetalTexture)); 
            r.leftThigh.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftThigh.RotationPoint = new Point3D(r.leftThigh.Cuboid.Rect3D.X + r.leftThigh.Cuboid.Rect3D.SizeX / 2,
                r.leftThigh.Cuboid.Rect3D.Y + r.leftThigh.Cuboid.Rect3D.SizeY, r.leftThigh.Cuboid.Rect3D.Z + r.leftThigh.Cuboid.Rect3D.SizeZ / 2);
            r.leftThigh.MinimumAngle = -25; r.leftThigh.MaximumAngle = 100;
            r.leftThigh.AddRelatedPart(r.leftHip1);
            r.leftThigh.AddRelatedPart(r.leftHip2);

            r.rightHip1.RotationPoint = r.rightThigh.RotationPoint;
            r.leftHip1.RotationPoint = r.leftThigh.RotationPoint;
            r.rightHip2.RotationPoint = r.rightThigh.RotationPoint;
            r.leftHip2.RotationPoint = r.leftThigh.RotationPoint;

            // Lytka
            r.rightShank = new Robot3DPart(r, "Pravé lýtko", (int)JointTypes.RightShank);
            r.rightShank.Code = "RLE4";
            r.rightShank.Cuboid = new Cuboid(new Rect3D(-0.095, -0.335, -0.03, 0.0800, 0.110, 0.0700), new ImageBrush(Images.MetalTexture));
            r.rightShank.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightShank.RotationPoint = new Point3D(r.rightShank.Cuboid.Rect3D.X,
                r.rightShank.Cuboid.Rect3D.Y + r.rightShank.Cuboid.Rect3D.SizeY, r.rightShank.Cuboid.Rect3D.Z + r.rightShank.Cuboid.Rect3D.SizeZ / 2);
            r.rightShank.MinimumAngle = -130; r.rightShank.MaximumAngle = 1;

            r.leftShank = new Robot3DPart(r, "Ľavé lýtko", (int)JointTypes.LeftShank);
            r.leftShank.Code = "LLE4";
            r.leftShank.Cuboid = new Cuboid(new Rect3D(0.015, -0.335, -0.03, 0.0800, 0.1100, 0.0700), new ImageBrush(Images.MetalTexture)); 
            r.leftShank.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftShank.RotationPoint = new Point3D(r.leftShank.Cuboid.Rect3D.X,
                r.leftShank.Cuboid.Rect3D.Y + r.leftShank.Cuboid.Rect3D.SizeY, r.leftShank.Cuboid.Rect3D.Z + r.leftShank.Cuboid.Rect3D.SizeZ / 2);
            r.leftShank.MinimumAngle = -130; r.leftShank.MaximumAngle = 1;

            // Členky
            r.rightAnkle = new Robot3DPart(r, "Pravý členok", (int)JointTypes.RightAnkle);
            r.rightAnkle.Code = "RLE5";
            r.rightAnkle.Cuboid = new Cuboid(new Rect3D(-0.065, -0.345, -0.015, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            r.rightAnkle.RotationAxis = new Vector3D(-1, 0, 0);
            r.rightAnkle.MinimumAngle = -45; r.rightAnkle.MaximumAngle = 75;

            r.leftAnkle = new Robot3DPart(r, "Ľavý členok", (int)JointTypes.LeftAnkle);
            r.leftAnkle.Code = "LLE5";
            r.leftAnkle.Cuboid = new Cuboid(new Rect3D(0.045, -0.345, -0.015, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            r.leftAnkle.RotationAxis = new Vector3D(-1, 0, 0);
            r.leftAnkle.MinimumAngle = -45; r.leftAnkle.MaximumAngle = 75;


            // Chodidla
            r.rightFoot = new Robot3DPart(r, "Pravé chodidlo", (int)JointTypes.RightFoot);
            r.rightFoot.Code = "RLE6";
            r.rightFoot.Cuboid = new Cuboid(new Rect3D(-0.095, -0.385, -0.055, 0.0800, 0.0300, 0.1600), new ImageBrush(Images.MetalTexture));
            r.rightFoot.RotationAxis = new Vector3D(0, 0, 1);
            r.rightFoot.RotationPoint = new Point3D(r.rightFoot.Cuboid.Rect3D.X + r.rightFoot.Cuboid.Rect3D.SizeX / 2,
                r.rightFoot.Cuboid.Rect3D.Y + r.rightFoot.Cuboid.Rect3D.SizeY, r.rightFoot.Cuboid.Rect3D.Z);
            r.rightFoot.MinimumAngle = -25; r.rightFoot.MaximumAngle = 45;
            r.rightFoot.AddRelatedPart(r.rightAnkle);

            r.leftFoot = new Robot3DPart(r, "Ľavé chodidlo", (int)JointTypes.LeftFoot);
            r.leftFoot.Code = "LLE6";
            r.leftFoot.Cuboid = new Cuboid(new Rect3D(0.015, -0.385, -0.055, 0.0800, 0.0300, 0.1600), new ImageBrush(Images.MetalTexture)); 
            r.leftFoot.RotationAxis = new Vector3D(0, 0, 1);
            r.leftFoot.RotationPoint = new Point3D(r.leftFoot.Cuboid.Rect3D.X + r.leftFoot.Cuboid.Rect3D.SizeX / 2,
                r.leftFoot.Cuboid.Rect3D.Y + r.leftFoot.Cuboid.Rect3D.SizeY, r.leftFoot.Cuboid.Rect3D.Z);
            r.leftFoot.MinimumAngle = -45; r.leftFoot.MaximumAngle = 25;
            r.leftFoot.AddRelatedPart(r.leftAnkle);

            r.rightAnkle.RotationPoint = r.rightFoot.RotationPoint;
            r.leftAnkle.RotationPoint = r.leftFoot.RotationPoint;

            // Spoje
            r.head.JoinWith(r.neck);
            r.neck.JoinWith(r.torso);
            r.rightShoulder.JoinWith(r.torso); r.leftShoulder.JoinWith(r.torso);
            r.rightUpperArm.JoinWith(r.rightShoulder); r.leftUpperArm.JoinWith(r.leftShoulder);
            r.rightElbow.JoinWith(r.rightUpperArm); r.leftElbow.JoinWith(r.leftUpperArm);
            r.rightLowerArm.JoinWith(r.rightElbow); r.leftLowerArm.JoinWith(r.leftElbow);
            r.rightHip1.JoinWith(r.torso); r.leftHip1.JoinWith(r.torso);
            r.rightHip2.JoinWith(r.rightHip1); r.leftHip2.JoinWith(r.leftHip1);
            r.rightThigh.JoinWith(r.rightHip2); r.leftThigh.JoinWith(r.leftHip2);
            r.rightShank.JoinWith(r.rightThigh); r.leftShank.JoinWith(r.leftThigh);
            r.rightAnkle.JoinWith(r.rightShank); r.leftAnkle.JoinWith(r.leftShank);
            r.rightFoot.JoinWith(r.rightAnkle); r.leftFoot.JoinWith(r.leftAnkle);

            return r;
        }

        public static Robot3D GetRobotFromFile(string fileName)
        {            
            using (TextReader tr = new StreamReader(fileName))
            {
                Robot3D retval;
                XmlSerializer xs = new XmlSerializer(typeof(Robot3D));
                retval = (Robot3D)xs.Deserialize(tr);

                foreach (var part in retval.Parts)
                {
                    part.Robot = retval;

                    foreach (var dp in part.SerializedDependingParts)
                    {
                        part.DependingParts.Add(GetPartByCode(retval, dp));
                    }

                    foreach (var rp in part.SerializedRelatedParts)
                    {
                        part.RelatedParts.Add(GetPartByCode(retval, rp));
                    }

                    part.Parent = GetPartByCode(retval, part.SerializedParent);
                }

                return retval;
            }
        }

        public static Robot3DPart GetPartByCode(Robot3D robot, string partCode)
        {
            foreach (var part in robot.Parts)
            {
                if (part.Code == partCode)
                    return part;
            }
            return null;
        }
        
        public void Save(string fileName)
        {
            using (TextWriter tw = new StreamWriter(fileName))
            {
                XmlSerializer xs = new XmlSerializer(typeof(Robot3D));
                xs.Serialize(tw, this);
            }            
        }        

        public void InsertToViewport(Viewport3D viewport)
        {
            foreach (Robot3DPart part in Parts)
                part.InsertToViewport(viewport);

            if (viewport.Camera is PerspectiveCamera)
            {
                ((PerspectiveCamera)viewport.Camera).Position = DefaultCameraPosition;
            }
        }

        public void AddPart(Robot3DPart part)
        {
            if (!parts.ContainsKey(part.Id))
                parts.Add(part.Id, part);
        }

        public Robot3DPart GetPartWithId(int id)
        {
            if (parts.ContainsKey(id))
                return parts[id];
            else
                return null;
        }

        #region IDisposable Members

        public void Dispose()
        {
            foreach (Robot3DPart part in Parts)
                part.Dispose();
        }

        #endregion

        public void UpdatePartPositions()
        {
            foreach (Robot3DPart part in Parts)
            {
                part.UpdateTransform();
            }
        }

        private void SetRobotState(Robot3DState state, Robot3DControl control)
        {
            if (control.Dispatcher.CheckAccess())
            {
                SetRobotState(state);
            }
            else
            {
                control.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Send,
                   new Action(
                     delegate()
                     {
                         SetRobotState(state);
                     }
                 ));
            }  

            
        }

        public void SetRobotState(Robot3DState state)
        {
            foreach (var ja in state.JointAngles)
            {
                if (parts[ja.Key].CurrentAngle != ja.Value)
                    parts[ja.Key].SetRotation(ja.Value);
            }

            UpdateCenterOfMass();
        }

        private void UpdateCenterOfMass()
        {
            if (Program.Config.VisualisationCenterOfMass)
            {
                Point3D centerOfMass = new PhysicalModel(this, new Gyroscope(0, 0, 0)).CenterOfMass;

                double tmp = centerOfMass.Z;
                centerOfMass.Z = centerOfMass.Y;
                centerOfMass.Y = tmp;
                centerOfMass.Offset(-0.05, -0.05, -0.05);

                centerOfMassCuboid.Rect3D = new Rect3D(centerOfMass, new Size3D(0.1, 0.1, 0.1));
                //centerOfMassCuboid.InsertToViewport(robotViewport, "", -1);
            }
        }
       
    }
}
