﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.Windows.Controls;
using RobotBehaviourEditor.Model;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{
    public class Robot3D : IDisposable
    {
        #region Robot Parts (Properties)
        private Dictionary<int, Robot3DPart> parts = new Dictionary<int, Robot3DPart>(16);
        public IList<Robot3DPart> Parts
        {
            get { return parts.Select(x => x.Value).ToList(); }            
        }

        private Robot3DPart head;
        public Robot3DPart Head
        {
            get { return head; }            
        }

        private Robot3DPart neck;
        public Robot3DPart Neck
        {
            get { return neck; }            
        }

        private Robot3DPart torso;
        public Robot3DPart Torso
        {
            get { return torso; }       
        }

        private Robot3DPart leftShoulder;
        public Robot3DPart LeftShoulder
        {
            get { return leftShoulder; }
        }

        private Robot3DPart rightShoulder;
        public Robot3DPart RightShoulder
        {
            get { return rightShoulder; }
        }

        private Robot3DPart leftUpperArm;
        public Robot3DPart LeftUpperArm
        {
            get { return leftUpperArm; }            
        }

        private Robot3DPart rightUpperArm;
        public Robot3DPart RightUpperArm
        {
            get { return rightUpperArm; }            
        }

        private Robot3DPart leftElbow;
        public Robot3DPart LeftElbow
        {
            get { return leftElbow; }
        }

        private Robot3DPart rightElbow;
        public Robot3DPart RightElbow
        {
            get { return rightElbow; }
        }

        private Robot3DPart leftLowerArm;
        public Robot3DPart LeftLowerArm
        {
            get { return leftLowerArm; }            
        }

        private Robot3DPart rightLowerArm;
        public Robot3DPart RightLowerArm
        {
            get { return rightLowerArm; }            
        }

        private Robot3DPart leftHip1;
        public Robot3DPart LeftHip1
        {
            get { return leftHip1; } 
        }

        private Robot3DPart rightHip1;
        public Robot3DPart RightHip1
        {
            get { return rightHip1; }
        }

        private Robot3DPart leftHip2;
        public Robot3DPart LeftHip2
        {
            get { return leftHip2; }
        }

        private Robot3DPart rightHip2;
        public Robot3DPart RightHip2
        {
            get { return rightHip2; }
        }

        private Robot3DPart leftThigh;
        public Robot3DPart LeftThigh
        {
            get { return leftThigh; }
        }

        private Robot3DPart rightThigh;
        public Robot3DPart RightThigh
        {
            get { return rightThigh; }
        }

        private Robot3DPart leftShank;
        public Robot3DPart LeftShank
        {
            get { return leftShank; }            
        }

        private Robot3DPart rightShank;
        public Robot3DPart RightShank
        {
            get { return rightShank; }            
        }

        private Robot3DPart leftAnkle;
        public Robot3DPart LeftAnkle
        {
            get { return leftAnkle; }
        }

        private Robot3DPart rightAnkle;
        public Robot3DPart RightAnkle
        {
            get { return rightAnkle; }
        }

        private Robot3DPart leftFoot;
        public Robot3DPart LeftFoot
        {
            get { return leftFoot; }        
        }

        private Robot3DPart rightFoot;
        public Robot3DPart RightFoot
        {
            get { return rightFoot; }
        }
        #endregion

        public Robot3D()
        {
            // Hlava
            head = new Robot3DPart(this, "Hlava", (int)JointTypes.Head);
            head.Cuboid = new Cuboid(new Rect3D(0.115, 0.31, 0.115, 0.1300, 0.1300, 0.1300), new ImageBrush(Images.FaceTexture));
            head.RotationAxis = new Vector3D(-1, 0, 0);
            head.RotationPoint = new Point3D(head.Cuboid.Rect3D.X + head.Cuboid.Rect3D.SizeX / 2, head.Cuboid.Rect3D.Y, head.Cuboid.Rect3D.Z + head.Cuboid.Rect3D.SizeZ / 2);
            head.MinimumAngle = -45; head.MaximumAngle = 45;

            // Krk
            neck = new Robot3DPart(this, "Krk", (int)JointTypes.Neck);
            neck.Cuboid = new Cuboid(new Rect3D(0.065, 0.22, 0.065, 0.0300, 0.0800, 0.0300), new ImageBrush(Images.MetalTexture));
            neck.RotationAxis = new Vector3D(0, -1, 0);
            neck.RotationPoint = new Point3D(neck.Cuboid.Rect3D.X + neck.Cuboid.Rect3D.SizeX/2, neck.Cuboid.Rect3D.Y, neck.Cuboid.Rect3D.Z + neck.Cuboid.Rect3D.SizeZ / 2);
            neck.MinimumAngle = -120; neck.MaximumAngle = 120;

            // Trup
            torso = new Robot3DPart(this, "Trup", (int)JointTypes.Torso);
            torso.Cuboid = new Cuboid(new Rect3D(0.05, 0.09, 0.05, 0.1000, 0.1800, 0.1000), new ImageBrush(Images.MetalTexture)); 
            torso.RotationPoint = new Point3D(torso.Cuboid.Rect3D.X, torso.Cuboid.Rect3D.Y, torso.Cuboid.Rect3D.Z + torso.Cuboid.Rect3D.SizeZ / 2);

            // Ruky
            // Plecia
            rightShoulder = new Robot3DPart(this, "Pravé plece", (int)JointTypes.RightShoulder);
            rightShoulder.Cuboid = new Cuboid(new Rect3D(0.158, 0.175, 0.06, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            rightShoulder.RotationAxis = new Vector3D(-1, 0, 0);
            rightShoulder.MinimumAngle = -120; rightShoulder.MaximumAngle = 120;

            leftShoulder = new Robot3DPart(this, "Ľavé plece", (int)JointTypes.LeftShoulder);
            leftShoulder.Cuboid = new Cuboid(new Rect3D(-0.038, 0.175, 0.06, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));  
            leftShoulder.RotationAxis = new Vector3D(-1, 0, 0);
            leftShoulder.MinimumAngle = -120; leftShoulder.MaximumAngle = 120;

            // Ramena
            rightUpperArm = new Robot3DPart(this, "Pravé rameno", (int)JointTypes.RightUpperArm);
            rightUpperArm.Cuboid = new Cuboid(new Rect3D(0.193, 0.195, 0.11, 0.0700, 0.0600, 0.0800), new ImageBrush(Images.MetalTexture));
            rightUpperArm.RotationAxis = new Vector3D(0, 1, 0);
            rightUpperArm.RotationPoint = new Point3D(rightUpperArm.Cuboid.Rect3D.X + rightUpperArm.Cuboid.Rect3D.SizeX,
                rightUpperArm.Cuboid.Rect3D.Y + rightUpperArm.Cuboid.Rect3D.SizeY / 2, rightUpperArm.Cuboid.Rect3D.Z + rightUpperArm.Cuboid.Rect3D.SizeY / 2);
            rightUpperArm.MinimumAngle = -95; rightUpperArm.MaximumAngle = 1;
            rightUpperArm.AddRelatedPart(rightShoulder);

            leftUpperArm = new Robot3DPart(this, "Ľavé rameno", (int)JointTypes.LeftUpperArm);
            leftUpperArm.Cuboid = new Cuboid(new Rect3D(-0.023, 0.195, 0.11, 0.0700, 0.0600, 0.0800), new ImageBrush(Images.MetalTexture));            
            leftUpperArm.RotationAxis = new Vector3D(0, 1, 0);
            leftUpperArm.RotationPoint = new Point3D(leftUpperArm.Cuboid.Rect3D.X,
                leftUpperArm.Cuboid.Rect3D.Y + leftUpperArm.Cuboid.Rect3D.SizeY / 2, leftUpperArm.Cuboid.Rect3D.Z + rightUpperArm.Cuboid.Rect3D.SizeY / 2);
            leftUpperArm.MinimumAngle = -1; leftUpperArm.MaximumAngle = 95;
            leftUpperArm.AddRelatedPart(leftShoulder);
            
            rightShoulder.RotationPoint = rightUpperArm.RotationPoint;
            leftShoulder.RotationPoint = leftUpperArm.RotationPoint;

            // Lakte
            rightElbow = new Robot3DPart(this, "Pravý lakeť", (int)JointTypes.RightElbow);
            rightElbow.Cuboid = new Cuboid(new Rect3D(0.158, 0.184, 0.15, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            rightElbow.RotationAxis = new Vector3D(0, 0, 1);
            rightElbow.MinimumAngle = -120; rightElbow.MaximumAngle = 120;

            leftElbow = new Robot3DPart(this, "Ľavý lakeť", (int)JointTypes.LeftElbow);
            leftElbow.Cuboid = new Cuboid(new Rect3D(-0.038, 0.184, 0.15, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            leftElbow.RotationAxis = new Vector3D(0, 0, 1);
            leftElbow.MinimumAngle = -120; leftElbow.MaximumAngle = 120;

            // Predlaktia
            rightLowerArm = new Robot3DPart(this, "Pravé predlaktie", (int)JointTypes.RightLowerArm);
            rightLowerArm.Cuboid = new Cuboid(new Rect3D(0.173, 0.199, 0.245, 0.0500, 0.0500, 0.1100), new ImageBrush(Images.MetalTexture));            
            rightLowerArm.RotationAxis = new Vector3D(0, 1, 0);
            rightLowerArm.RotationPoint = new Point3D(rightLowerArm.Cuboid.Rect3D.X + rightLowerArm.Cuboid.Rect3D.SizeX / 2,
                rightLowerArm.Cuboid.Rect3D.Y + rightLowerArm.Cuboid.Rect3D.SizeY/2, rightLowerArm.Cuboid.Rect3D.Z);            
            rightLowerArm.MinimumAngle = -1; rightLowerArm.MaximumAngle = 90;
            rightLowerArm.AddRelatedPart(rightElbow);

            leftLowerArm = new Robot3DPart(this, "Ľavé predlaktie", (int)JointTypes.LeftLowerArm);
            leftLowerArm.Cuboid = new Cuboid(new Rect3D(-0.023, 0.199, 0.245, 0.0500, 0.0500, 0.1100), new ImageBrush(Images.MetalTexture));            
            leftLowerArm.RotationAxis = new Vector3D(0, 1, 0);
            leftLowerArm.RotationPoint = new Point3D(leftLowerArm.Cuboid.Rect3D.X + leftLowerArm.Cuboid.Rect3D.SizeX / 2,
                leftLowerArm.Cuboid.Rect3D.Y + leftLowerArm.Cuboid.Rect3D.SizeY/2, leftLowerArm.Cuboid.Rect3D.Z);            
            leftLowerArm.MinimumAngle = -90; leftLowerArm.MaximumAngle = 1;
            leftLowerArm.AddRelatedPart(leftElbow);

            rightElbow.RotationPoint = rightLowerArm.RotationPoint;
            leftElbow.RotationPoint = leftLowerArm.RotationPoint;

            // Boky
            rightHip1 = new Robot3DPart(this, "Pravý bok 1", (int)JointTypes.RightHip1);
            rightHip1.Cuboid = new Cuboid(new Rect3D(0.115, -0.015, 0.05, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            rightHip1.RotationAxis = new Vector3D(0.7071, 0.7071, 0);
            rightHip1.MinimumAngle = -90; rightHip1.MaximumAngle = 1;

            leftHip1 = new Robot3DPart(this, "Ľavý bok 1", (int)JointTypes.LeftHip1);
            leftHip1.Cuboid = new Cuboid(new Rect3D(0.005, -0.015, 0.05, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            leftHip1.RotationAxis = new Vector3D(0.7071, -0.7071, 0);
            leftHip1.MinimumAngle = -90; leftHip1.MaximumAngle = 1;

            rightHip2 = new Robot3DPart(this, "Pravý bok 2", (int)JointTypes.RightHip2);
            rightHip2.Cuboid = new Cuboid(new Rect3D(0.115, -0.015, 0.05, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            rightHip2.RotationAxis = new Vector3D(0, 0, 1);           
            rightHip2.MinimumAngle = -45; rightHip2.MaximumAngle = 25;

            leftHip2 = new Robot3DPart(this, "Ľavý bok 2", (int)JointTypes.LeftHip2);
            leftHip2.Cuboid = new Cuboid(new Rect3D(0.005, -0.015, 0.05, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            leftHip2.RotationAxis = new Vector3D(0, 0, 1);
            leftHip2.MinimumAngle = -25; leftHip2.MaximumAngle = 45;

            // Nohy
            // Stehna
            rightThigh = new Robot3DPart(this, "Pravé stehno", (int)JointTypes.RightThigh);
            rightThigh.Cuboid = new Cuboid(new Rect3D(0.14, 0.005, 0.085, 0.0700, 0.1400, 0.0700), new ImageBrush(Images.MetalTexture));
            rightThigh.RotationAxis = new Vector3D(-1, 0, 0);
            rightThigh.RotationPoint = new Point3D(rightThigh.Cuboid.Rect3D.X + rightThigh.Cuboid.Rect3D.SizeX / 2,
                rightThigh.Cuboid.Rect3D.Y + rightThigh.Cuboid.Rect3D.SizeY, rightThigh.Cuboid.Rect3D.Z + rightThigh.Cuboid.Rect3D.SizeZ / 2);
            rightThigh.MinimumAngle = -25; rightThigh.MaximumAngle = 100;
            rightThigh.AddRelatedPart(rightHip1);
            rightThigh.AddRelatedPart(rightHip2);

            leftThigh = new Robot3DPart(this, "Ľavé stehno", (int)JointTypes.LeftThigh);
            leftThigh.Cuboid = new Cuboid(new Rect3D(0.03, 0.005, 0.085, 0.0700, 0.1400, 0.0700), new ImageBrush(Images.MetalTexture));
            leftThigh.RotationAxis = new Vector3D(-1, 0, 0);
            leftThigh.RotationPoint = new Point3D(leftThigh.Cuboid.Rect3D.X + leftThigh.Cuboid.Rect3D.SizeX / 2,
                leftThigh.Cuboid.Rect3D.Y + leftThigh.Cuboid.Rect3D.SizeY, leftThigh.Cuboid.Rect3D.Z + leftThigh.Cuboid.Rect3D.SizeZ / 2);
            leftThigh.MinimumAngle = -25; leftThigh.MaximumAngle = 100;
            leftThigh.AddRelatedPart(leftHip1);
            leftThigh.AddRelatedPart(leftHip2);

            rightHip1.RotationPoint = rightThigh.RotationPoint;
            leftHip1.RotationPoint = leftThigh.RotationPoint;
            rightHip2.RotationPoint = rightThigh.RotationPoint;
            leftHip2.RotationPoint = leftThigh.RotationPoint;            

            // Lytka
            rightShank = new Robot3DPart(this, "Pravé lýtko", (int)JointTypes.RightShank);
            rightShank.Cuboid = new Cuboid(new Rect3D(0.145, -0.135, 0.09, 0.0800, 0.1100, 0.0700), new ImageBrush(Images.MetalTexture));
            rightShank.RotationAxis = new Vector3D(-1, 0, 0);
            rightShank.RotationPoint = new Point3D(rightShank.Cuboid.Rect3D.X,
                rightShank.Cuboid.Rect3D.Y + rightShank.Cuboid.Rect3D.SizeY, rightShank.Cuboid.Rect3D.Z + rightShank.Cuboid.Rect3D.SizeZ / 2);
            rightShank.MinimumAngle = -130; rightShank.MaximumAngle = 1;

            leftShank = new Robot3DPart(this, "Ľavé lýtko", (int)JointTypes.LeftShank);
            leftShank.Cuboid = new Cuboid(new Rect3D(0.035, -0.135, 0.09, 0.0800, 0.1100, 0.0700), new ImageBrush(Images.MetalTexture));
            leftShank.RotationAxis = new Vector3D(-1, 0, 0);
            leftShank.RotationPoint = new Point3D(leftShank.Cuboid.Rect3D.X,
                leftShank.Cuboid.Rect3D.Y + leftShank.Cuboid.Rect3D.SizeY, leftShank.Cuboid.Rect3D.Z + leftShank.Cuboid.Rect3D.SizeZ / 2);
            leftShank.MinimumAngle = -130; leftShank.MaximumAngle = 1;

            // Členky
            rightAnkle = new Robot3DPart(this, "Pravý členok", (int)JointTypes.RightAnkle);
            rightAnkle.Cuboid = new Cuboid(new Rect3D(0.115, -0.235, 0.055, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture)); 
            rightAnkle.RotationAxis = new Vector3D(-1, 0, 0);
            rightAnkle.MinimumAngle = -45; rightAnkle.MaximumAngle = 75;

            leftAnkle = new Robot3DPart(this, "Ľavý členok", (int)JointTypes.LeftAnkle);
            leftAnkle.Cuboid = new Cuboid(new Rect3D(0.005, -0.235, 0.055, 0.0200, 0.0200, 0.0200), new ImageBrush(Images.MetalTexture));
            leftAnkle.RotationAxis = new Vector3D(-1, 0, 0);
            leftAnkle.MinimumAngle = -45; leftAnkle.MaximumAngle = 75;


            // Chodidla
            rightFoot = new Robot3DPart(this, "Pravé chodidlo", (int)JointTypes.RightFoot);
            rightFoot.Cuboid = new Cuboid(new Rect3D(0.145, -0.265, 0.155, 0.0800, 0.0300, 0.1600), new ImageBrush(Images.MetalTexture));
            rightFoot.RotationAxis = new Vector3D(0, 0, 1);
            rightFoot.RotationPoint = new Point3D(rightFoot.Cuboid.Rect3D.X + rightFoot.Cuboid.Rect3D.SizeX / 2,
                rightFoot.Cuboid.Rect3D.Y + rightFoot.Cuboid.Rect3D.SizeY, rightFoot.Cuboid.Rect3D.Z);            
            rightFoot.MinimumAngle = -25; rightFoot.MaximumAngle = 45;
            rightFoot.AddRelatedPart(rightAnkle);

            leftFoot = new Robot3DPart(this, "Ľavé chodidlo", (int)JointTypes.LeftFoot);
            leftFoot.Cuboid = new Cuboid(new Rect3D(0.035, -0.265, 0.155, 0.0800, 0.0300, 0.1600), new ImageBrush(Images.MetalTexture));
            leftFoot.RotationAxis = new Vector3D(0, 0, 1);
            leftFoot.RotationPoint = new Point3D(leftFoot.Cuboid.Rect3D.X + leftFoot.Cuboid.Rect3D.SizeX/2,
                leftFoot.Cuboid.Rect3D.Y + leftFoot.Cuboid.Rect3D.SizeY, leftFoot.Cuboid.Rect3D.Z);            
            leftFoot.MinimumAngle = -45; leftFoot.MaximumAngle = 25;
            leftFoot.AddRelatedPart(leftAnkle);

            rightAnkle.RotationPoint = rightFoot.RotationPoint;
            leftAnkle.RotationPoint = leftFoot.RotationPoint;

            // Spoje
            head.JoinWith(neck);
            neck.JoinWith(torso);
            rightShoulder.JoinWith(torso); leftShoulder.JoinWith(torso);
            rightUpperArm.JoinWith(rightShoulder); leftUpperArm.JoinWith(leftShoulder);
            rightElbow.JoinWith(rightUpperArm); leftElbow.JoinWith(leftUpperArm);
            rightLowerArm.JoinWith(rightElbow); leftLowerArm.JoinWith(leftElbow);
            rightHip1.JoinWith(torso); leftHip1.JoinWith(torso);
            rightHip2.JoinWith(rightHip1); leftHip2.JoinWith(leftHip1);
            rightThigh.JoinWith(rightHip2); leftThigh.JoinWith(leftHip2);
            rightShank.JoinWith(rightThigh); leftShank.JoinWith(leftThigh);
            rightAnkle.JoinWith(rightShank); leftAnkle.JoinWith(leftShank);
            rightFoot.JoinWith(rightAnkle); leftFoot.JoinWith(leftAnkle);            
        }

        public void InsertToViewport(Viewport3D viewport)
        {
            foreach (Robot3DPart part in Parts)
                part.InsertToViewport(viewport);
        }

        public void AddPart(Robot3DPart part)
        {
            if (!parts.ContainsKey(part.Id))
                parts.Add(part.Id, part);
        }

        public Robot3DPart GetPartWithId(int id)
        {
            if (parts.ContainsKey(id))
                return parts[id];
            else
                return null;
        }

        #region IDisposable Members

        public void Dispose()
        {
            foreach (Robot3DPart part in Parts)
                part.Dispose();
        }

        #endregion

        public void UpdatePartPositions()
        {
            foreach (Robot3DPart part in Parts)
            {
                part.UpdateTransform();
            }
        }

        private void SetRobotState(Robot3DState state, Robot3DControl control)
        {
            if (control.Dispatcher.CheckAccess())
            {
                SetRobotState(state);
            }
            else
            {
                control.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Send,
                   new Action(
                     delegate()
                     {
                         SetRobotState(state);
                     }
                 ));
            }  

            
        }

        public void SetRobotState(Robot3DState state)
        {
            foreach (var ja in state.JointAngles)
            {
                if (parts[ja.Key].CurrentAngle != ja.Value)
                    parts[ja.Key].SetRotation(ja.Value);
            }
        }
        
    }
}
