﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Media3D;
using System.Windows.Controls;
using System.Windows.Media;
using System.Xml.Serialization;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{
    [XmlRoot("Robot3DPart")]
    public class Robot3DPart : IDisposable
    {
        #region Properties
        private Robot3D robot = null;
        [XmlIgnore()]
        public Robot3D Robot
        {
            get { return robot; }
            internal set { robot = value; }
        }
        
        private string name;
        [XmlAttribute("Name")]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        private string code;
        [XmlAttribute("Code")]
        public string Code
        {
            get { return code; }
            set { code = value; }
        }

        private int id;
        [XmlAttribute("Id")]
        public int Id
        {
            get { return id; }
            set { id = value; }
        }

        private Cuboid cuboid;
        [XmlElement("Cuboid")]
        public Cuboid Cuboid
        {
            get { return cuboid; }
            set
            {
                cuboid = value;
                if (value != null)
                    Transform = cuboid.Model3D.Transform;
            }
        }

        // Na ktorej casti zavisi tato cast        
        private Robot3DPart parent;
        [XmlIgnore()]        
        public Robot3DPart Parent
        {
            get { return parent; }
            internal set { parent = value; }
        }

        // Casti, ktore su zavisle na tejto casti
        private List<Robot3DPart> dependingParts = new List<Robot3DPart>();
        [XmlIgnore()]        
        public List<Robot3DPart> DependingParts
        {
            get { return dependingParts; }            
        }

        // Casti, ktore sa tiez nastavuju oznacenim tejto casti
        private List<Robot3DPart> relatedParts = new List<Robot3DPart>();
        [XmlIgnore()]         
        public List<Robot3DPart> RelatedParts
        {
            get { return relatedParts; }
        }

        #region Serialization - Special elements
        private string serializedParent;
        [XmlAttribute("Parent")]
        public string SerializedParent
        {
            get
            {
                if (parent == null)
                    return "";
                else
                    return parent.Code;
            }
            set { serializedParent = value; }
        }

        private List<string> serializedRelatedParts = new List<string>();
        [XmlArray("RelatedParts"), XmlArrayItem("Part", typeof(string))]
        public List<string> SerializedRelatedParts
        {
            get { 
                return relatedParts.Select(x => x.code).ToList(); }
            set { serializedRelatedParts = value; }
        }

        private List<string> serializedDependingParts;
        [XmlArray("DependingParts"), XmlArrayItem("Part", typeof(string))]
        public List<string> SerializedDependingParts
        {
            get { return dependingParts.Select(x => x.code).ToList(); }
            set { serializedDependingParts = value; }
        }
        #endregion

        private Vector3D rotationAxis;
        [XmlElement("RotationAxis")]
        public Vector3D RotationAxis
        {
            get { return rotationAxis; }
            set
            {
                rotationAxis = value;
                TransformedRotationAxis = rotationAxis;
            }
        }

        private Point3D rotationPoint;
        [XmlElement("RotationPoint")]
        public Point3D RotationPoint
        {
            get { return rotationPoint; }
            set { rotationPoint = value; }
        }

        private double maximumAngle = 0;
        [XmlAttribute("MaximumAngle")]
        public double MaximumAngle
        {
            get { return maximumAngle; }
            set { maximumAngle = value; }
        }

        private double minimumAngle = 0;
        [XmlAttribute("MinimumAngle")]
        public double MinimumAngle
        {
            get { return minimumAngle; }
            set { minimumAngle = value; }
        }        

        private double currentAngle = 0;
        [XmlAttribute("CurrentAngle")]
        public double CurrentAngle
        {
            get { return currentAngle; }
            set { currentAngle = value; }
        }
        
        private Point3D transformedRotationPoint;
        [XmlIgnore()]
        public Point3D TransformedRotationPoint
        {
            get { return transformedRotationPoint; }
            set { transformedRotationPoint = value; }
        }

        private Vector3D transformedRotationAxis;
        [XmlIgnore()]
        public Vector3D TransformedRotationAxis
        {
            get { return transformedRotationAxis; }
            set { transformedRotationAxis = value; }
        }

        private Transform3D transform;
        [XmlIgnore()]
        public Transform3D Transform
        {
            get { return transform; }
            set
            {
                transform = value;
                UpdateTransform();
                TransformDependingParts();
            }
        }        
        #endregion

        private Robot3DPart()
        {
        }

        public Robot3DPart(Robot3D robot, string name, int id)
        {
            this.name = name;
            this.id = id;
            this.robot = robot;

            this.robot.AddPart(this);
        }

        public void JoinWith(Robot3DPart part)
        {
            this.Parent = part;
            part.dependingParts.Add(this);
        }

        public void AddRelatedPart(Robot3DPart part)
        {
            this.RelatedParts.Add(part);            
        }

        public void InsertToViewport(Viewport3D viewport)
        {
            if (cuboid != null)
                cuboid.InsertToViewport(viewport, this.Name, this.Id);
        }

        public Transform3DGroup GetDependentTransform(bool withMyTransform)
        {
            Transform3DGroup group = new Transform3DGroup();

            if (Parent != null)
            {
                Transform3DGroup dependent = Parent.GetDependentTransform(true);
                foreach (Transform3D t in dependent.Children)
                {
                    group.Children.Add(t);
                }
            }
            if (withMyTransform)
            {
                if (this.Transform != null)
                    group.Children.Add(this.Transform);
            }

            return group;
        }

        public Model3DGroup GetDependentGroup()
        {
            Model3DGroup group = new Model3DGroup();

            if (Parent != null)
            {
                Model3DGroup dependent = Parent.GetDependentGroup();
                foreach (Model3D t in dependent.Children)
                {
                    group.Children.Add(t);
                }
            }            
            group.Children.Add(this.cuboid.Model3D);

            return group;
        }

        public void UpdateTransform()
        {            
            this.Cuboid.Model3D.Transform = GetDependentTransform(true);
            this.transformedRotationPoint = GetDependentTransform(false).Transform(RotationPoint);
            this.transformedRotationAxis = GetDependentTransform(false).Transform(RotationAxis);            
            //transformedRotationPoint = JoinedWith.Cuboid.Model3D.Transform.Transform(RotationPoint);            
        }

        private void TransformDependingParts()
        {
            foreach (Robot3DPart part in dependingParts)
            {                
                part.SetRotation(part.CurrentAngle);
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            this.robot = null;
            this.cuboid = null;
            this.parent = null;
        }

        #endregion

        public void SetRotation(double angle)
        {
            if (angle > MaximumAngle) angle = MaximumAngle;
            if (angle < MinimumAngle) angle = MinimumAngle;                       

            this.UpdateTransform();

            this.CurrentAngle = angle;

            AxisAngleRotation3D rotation = new AxisAngleRotation3D(
                   this.TransformedRotationAxis,
                   angle);

            this.Transform = new RotateTransform3D(rotation, this.TransformedRotationPoint);
        }
    }
}
