﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RobotBehaviourEditor.Model;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{
    public class Robot3DState
    {        
        // TODO: Vylepsit!
        private readonly static Robot3DState startState = new Robot3DState(Robot3D.GetRobot());
        public static Robot3DState StartState
        {
            get { return new Robot3DState(Robot3D.GetRobot()); }
        }

        private Dictionary<int, double> jointAngles = new Dictionary<int, double>(24);
        public Dictionary<int, double> JointAngles
        {
            get
            {
                return jointAngles;
            }
        }

        private Robot3DState()
        {
        }

        public Robot3DState(Robot3DState state)
        {
            jointAngles = new Dictionary<int, double>(state.jointAngles);            
        }

        public Robot3DState(Robot3D robot)
        {
            foreach (var part in robot.Parts)
            {
                jointAngles.Add(part.Id, part.CurrentAngle);
            }
        }

        public static Robot3DState GetStateBetween(Robot3DState state1, Robot3DState state2, double coef)
        {
            Robot3DState retval = new Robot3DState();
            foreach (var jointAngle in state1.jointAngles)
            {
                double value = jointAngle.Value * (1 - coef) + state2.JointAngles[jointAngle.Key] * coef;
                retval.jointAngles.Add(jointAngle.Key, value);
            }

            return retval;
        }     

        public static Robot3DState GetState(Robot3DState startState, List<JointMotion> motions)
        {
            Robot3DState retval = new Robot3DState(startState);
            var orderedMotions = from m in motions
                                 orderby m.EndPosition
                                 select m;
            
            foreach (var motion in orderedMotions)
            {
                retval.JointAngles[(int)motion.Joint] = motion.EndPosition;
            }

            return retval;
        }

        public static List<JointMotion> GetJointMotions(Robot3DState fromState, Robot3DState toState, int startTime, int duration)
        {
            List<JointMotion> retval = new List<JointMotion>();
            foreach (var ja in fromState.jointAngles)
            {
                int i = ja.Key;
                if (Math.Abs( fromState.jointAngles[i] - toState.jointAngles[i]) > 0.9)
                {
                    retval.Add(new JointMotion((JointTypes)i, 
                        Math.Round(fromState.jointAngles[i]), 
                        Math.Round(toState.jointAngles[i]), 
                        duration, startTime, null));
                }
            }
            return retval;
        }
    }
}
