﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RobotBehaviourEditor.Model;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{
    public class RobotAnimation
    {
        private List<JointMotion> jointMotions;

        public RobotAnimation(List<JointMotion> jointMotions)
        {
            this.jointMotions = jointMotions;
        }

        private List<JointMotion> GetActiveMotions(int currentTime)
        {
            List<JointMotion> retval = new List<JointMotion>();
            foreach (var jm in jointMotions)
            {
                if (jm.StartTime <= currentTime && jm.StartTime + jm.Duration >= currentTime)
                {
                    retval.Add(jm);
                }
            }
            return retval;
        }

        private List<JointMotion> GetPreviousMotions(int currentTime)
        {
            List<JointMotion> retval = new List<JointMotion>();
            foreach (var jm in jointMotions)
            {
                if (jm.StartTime < currentTime && jm.StartTime + jm.Duration < currentTime)
                {
                    retval.Add(jm);
                }
            }
            return retval;
        }

        public JointMotion GetLastMotion()
        {
            var motion = jointMotions.OrderBy(x => x.StartTime + x.Duration).LastOrDefault();
            return motion;
        }

        public Robot3DState GetPartialStateInTime(Robot3DState startState, double timeOffset)
        {
            Robot3DState retval = new Robot3DState(startState);
            List<JointMotion> activeMotions = GetActiveMotions((int)timeOffset);

            foreach (var motion in activeMotions)
            {
                double pos = motion.GetJoinPositionForTime(timeOffset);

                retval.JointAngles[(int)motion.Joint] = pos;
            }

            return retval;
        }

        public Robot3DState GetPartialStateInTime(double timeOffset)
        {
            return GetPartialStateInTime(Robot3DState.StartState, timeOffset);
        }

        public Robot3DState GetStateInTime(Robot3DState startState, double timeOffset)
        {
            Robot3DState retval = new Robot3DState(startState);
            List<JointMotion> activeMotions = GetActiveMotions((int)timeOffset);
            List<JointMotion> previousMotions = GetPreviousMotions((int)timeOffset).OrderBy(x => x.StartTime).ToList();

            foreach (var motion in previousMotions)
            {
                retval.JointAngles[(int)motion.Joint] = motion.EndPosition;
            }

            foreach (var motion in activeMotions)
            {
                double pos = motion.GetJoinPositionForTime(timeOffset);

                retval.JointAngles[(int)motion.Joint] = pos;
            }

            return retval;
        }

    }
}
