﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RobotBehaviourEditor.Model;
using System.Windows.Threading;

namespace RobotBehaviourEditor.MotionEditor.Visualisation
{
    public class RobotAnimator
    {
        private Robot3D robot;

        System.Windows.Controls.Slider animationSlider = null;

        private DispatcherTimer animationTimer = new DispatcherTimer(DispatcherPriority.Send);        
        private double startTime = 0;

        private Robot3DState robotState = Robot3DState.StartState;

        private List<JointMotion> jointMotions = new List<JointMotion>();

        private JointMotion lastMotion = null;

        private RobotAnimation animation = null;
        public RobotAnimation Animation
        {
            get { return animation; }
            private set { animation = value; }
        }

        private bool isBusy = false;
        public bool IsBusy
        {
            get { return isBusy; }
            private set { isBusy = value; }
        }

        public RobotAnimator(Robot3D robot)
        {
            this.robot = robot;
        }

        public void SetJointMotions(IList<JointMotion> jointMotions)
        {
            this.jointMotions = new List<JointMotion>(jointMotions);
            this.animation = new RobotAnimation(this.jointMotions);
        }        

        public void Start(Dispatcher dispatcher)
        {
            isBusy = true;
            lastMotion = animation.GetLastMotion();

            if (lastMotion != null)
            {
                startTime = TimeSpan.FromTicks(DateTime.Now.Ticks).TotalMilliseconds;

                animationTimer = new DispatcherTimer(DispatcherPriority.Send, dispatcher);
                animationTimer.Interval = new TimeSpan(0, 0, 0, 0, 20);
                animationTimer.Start();
                animationTimer.Tick += new EventHandler(animationTimer_Tick);

                robotState = Robot3DState.StartState;                

                if (animationSlider != null)
                {
                    animationSlider.Minimum = 0;
                    animationSlider.Maximum = (lastMotion.StartTime + lastMotion.Duration);
                }
            }
        }

        public void Stop()
        {            
            animationTimer.Stop();
            isBusy = false;
        }

        void animationTimer_Tick(object sender, EventArgs e)
        {
            double timeOffset = TimeSpan.FromTicks(DateTime.Now.Ticks).TotalMilliseconds - startTime;            

            // Posledny pohyb dokonceny, zastavenie vykonavania
            if (timeOffset > lastMotion.StartTime + lastMotion.Duration)
            {
                Stop();
                timeOffset = lastMotion.StartTime + lastMotion.Duration;
            }

            if (animationSlider != null)
                animationSlider.Value = Math.Min(timeOffset, animationSlider.Maximum);
            
            // Vykonanie pohybov
            robotState = animation.GetPartialStateInTime(robotState, timeOffset);

            robot.SetRobotState(robotState);
            
        }
        
        public void Start(Dispatcher dispatcher, System.Windows.Controls.Slider animationSlider)
        {            
            this.animationSlider = animationSlider;
            Start(dispatcher);
        }
    }
}
