﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using RobotBehaviourEditor.Model;

namespace RobotBehaviourEditor.ServerMessageParsing
{
    public class ServerMessageParser
    {

        private const int MAX_JOINT_S_EXPRESSION_LENGTH = 23;
        private const int SEEK_BACK_AFTER_JOINT_PARSED = 3;
        private const int ONE_MOTION_MILLISEC_DURATION = 20;

        private List<JointMotion> motions;

        private StreamReader reader;

        public ServerMessageParser(string fileName)
        {
            this.motions = new List<JointMotion>();

            try
            {
                reader = File.OpenText(fileName);
            }
            catch (FileNotFoundException)
            {
            }

        }

        public List<JointMotion> ParseMotions()
        {
            string line;

            string tmpJointData;
            string[] tmpJointDataSplitted;
            string tmpJointName;
            double tmpJointAngle;
            JointMotion tmpPreviousJointMotion;

            double startTime = -1;
            double actualTime = 0;
            string tmpTimeData;

            int parsedLineChars = 0;
            int currentJointPosition = 0;

            try
            {
                line = reader.ReadLine();

                while (line != null)
                {

                    // parsujeme cas (je na zaciatku riadku)
                    if (line.IndexOf("now") != -1) // kontrola, ci ide o riadok obsahujuci cas
                    {
                        tmpTimeData = line.Substring(line.IndexOf("now") + 3, line.IndexOf("GS") - line.IndexOf("now") - 6); // vysekneme udaj o case
                        actualTime = double.Parse(tmpTimeData.Replace('.', ','));
                        if (startTime == -1) // inicializujeme startovaci cas
                            startTime = actualTime;
                        //Console.WriteLine(++n + " " + actualTime);
                    }

                    parsedLineChars = 0;
                    currentJointPosition = 0;

                    while ((currentJointPosition = line.IndexOf("HJ", parsedLineChars)) != -1)
                    {
                        if (line[currentJointPosition] == 'h' && line[currentJointPosition + 1] == 'j')
                            // HJ a hj nie je to iste, parsujeme iba HJ
                            continue;

                        // ideme parsovat najdeny HJ
                        tmpJointData = line.Substring(currentJointPosition, MAX_JOINT_S_EXPRESSION_LENGTH);

                        // odstranime zatvorky
                        tmpJointData = tmpJointData.Replace('(', ' ');
                        tmpJointData = tmpJointData.Replace(')', ' ');

                        // rozdelime retazec
                        tmpJointDataSplitted = tmpJointData.Split(' ');

                        // z konkretnych pozicii vyberieme potrebne udaje
                        tmpJointName = tmpJointDataSplitted[3].Replace('j', 'e');
                        tmpJointAngle = double.Parse(tmpJointDataSplitted[7].Replace('.', ','));
                        tmpPreviousJointMotion = this.LastJointOccurance(Joints.GetJointTypeById(tmpJointName), int.MinValue);

                        if ((tmpPreviousJointMotion == null)
                            ||
                            (tmpPreviousJointMotion.EndPosition != tmpJointAngle))
                        {
                            // nebol zatial ziadny pohyb s danym klbom, zaradime ho do zoznamu
                            JointMotion newMotion =
                                new JointMotion(
                                    Joints.GetJointTypeById(tmpJointName),
                                // ak je klb nacitany prvy krat, startovacia aj koncova pozicia budu rovnake
                                    (tmpPreviousJointMotion == null ? 0 : tmpPreviousJointMotion.EndPosition),
                                    tmpJointAngle,
                                    ONE_MOTION_MILLISEC_DURATION,
                                    (int)Math.Round((actualTime - startTime) * 1000),
                                // dalsi pohyb daneho klbu zacneme hned po skonceni posledneho - pohyby nadvezuju;
                                // cas zo servera je v milisekundach, ma 2 desatinne miesta
                                    null);

                            motions.Add(newMotion);
                        }

                        // naparsovane znaky ziskame spocitanim aktualnej pozicie a maximalnej dlzky s-vyrazu pre HJ;
                        // odpocitame konstantu, aby sme v pripade ze sme zasiahli pri parsovani do nasledovneho HJ, tento
                        // nepreskocili, ale spravne naparsovali v dalsej iteracii cyklu
                        parsedLineChars = currentJointPosition + MAX_JOINT_S_EXPRESSION_LENGTH - SEEK_BACK_AFTER_JOINT_PARSED;

                        if (parsedLineChars >= line.Length)
                            // sparsovali sme aktualny riadok
                            break;

                    }

                    line = reader.ReadLine();

                }
                reader.Close();

            }

            catch (IOException)
            {
            }

            /* #DEBUG# LOGOVANIE

            // create a writer and open the file
            TextWriter tw = new StreamWriter("log.txt");

            for (int i = 0; i < this.motions.Count; i++)
            {

                tw.WriteLine(i + ". Joint: " + this.motions[i].Joint +
                                  " StartA: " + this.motions[i].StartPosition +
                                  " EndA: " + this.motions[i].EndPosition +
                                  " StartT: " + this.motions[i].StartTime +
                                  " Dur: " + this.motions[i].Duration);
                //if (i % 100 == 0)
                //    Console.ReadLine();


            }

            tw.Close();
            
            Console.ReadLine();
            
            */

            return motions;

        }

        private JointMotion LastJointOccurance(JointTypes joint, int initialIndex)
        {
            if (initialIndex == int.MinValue)
                initialIndex = this.motions.Count - 1;

            for (int i = initialIndex; i > 0; i--)
            {

                if (this.motions[i].Joint == joint)
                    return this.motions[i]; // v historii sme nasli pohyb ktory bol
                // vykonany nad danym klbom

            }

            return null; // nenasli sme pohyb nad danym klbom

        }

    }

}
