﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MathNet.Numerics.LinearAlgebra;

namespace RobotBehaviourEditor.Utils
{
    class Line
    {
        public double alfa = 0;
        public double beta = 0;
        public double gama = 0;
        private double[][] matDataB = new double[4][];
        private double[][] matDataA = new double[4][];
        public double[][] abc = new double[4][];
        private Matrix matA;
        private Matrix matB;
        private Matrix matC;

        private void init()
        {
            for (int i = 0; i < 4; i++)
            {
                matDataB[i] = new double[1];
                abc[i] = new double[1];
            }
            for (int i = 0; i < 4; i++)
            {
                matDataA[i] = new double[4];
            }
        }

        private Line()
        {
        }
        public Line(double[] point1, double[] point2)
        {
            init();
            if (point1[0] == point2[0] && point1[1] == point2[1])
            {
                this.alfa = 0;
                this.beta = 0;
                this.gama = 1;

            }
            else
            {
                if (point1[0] == point2[0] && point1[2] == point2[2])
                {
                    this.alfa = 0;
                    this.beta = 1;
                    this.gama = 0;
                }
                else
                {
                    if (point1[0] == point2[0])//mam problem
                    {
                        this.alfa = 0;
                        this.beta = Math.Cos(Math.Abs(Math.Atan((point1[1] - point2[1]) / (point1[2] - point2[2]))));
                        this.gama = Math.Cos(Math.Abs(Math.Atan((point1[2] - point2[2]) / (point1[1] - point2[1]))));
                    }
                    else
                    {
                        matDataB[0][0] = point1[1];
                        matDataB[1][0] = point1[2];
                        matDataB[2][0] = point2[1];
                        matDataB[3][0] = point2[2];

                        matDataA[0][0] = point1[0];
                        matDataA[0][1] = 1;
                        matDataA[0][2] = 0;
                        matDataA[0][3] = 0;
                        matDataA[1][0] = 0;
                        matDataA[1][1] = 0;
                        matDataA[1][2] = point1[0];
                        matDataA[1][3] = 1;
                        matDataA[2][0] = point2[0];
                        matDataA[2][1] = 1;
                        matDataA[2][2] = 0;
                        matDataA[2][3] = 0;
                        matDataA[3][0] = 0;
                        matDataA[3][1] = 0;
                        matDataA[3][2] = point2[0];
                        matDataA[3][3] = 1;

                        matA = new Matrix(matDataA);
                        matB = new Matrix(matDataB);

                        matC = matA.Solve(matB);
                        this.abc = matC.GetArray();
                        this.alfa = 1 / Math.Sqrt(1 + abc[0][0] * abc[0][0] + abc[2][0] * abc[2][0]);
                        this.beta = abc[0][0] / Math.Sqrt(1 + abc[0][0] * abc[0][0] + abc[2][0] * abc[2][0]);
                        this.gama = abc[2][0] / Math.Sqrt(1 + abc[0][0] * abc[0][0] + abc[2][0] * abc[2][0]);
                    }
                }
            }
        }
        public Line(Plain plain)
        {
            this.alfa = plain.a;
            this.beta = plain.b;
            this.gama = plain.c;
        }

        public Line(Plain plain1, Plain plain2)
        {
            init();
            double param;
            double det1;
            double det2;
            double det3;
            det1 = plain1.b * plain2.c - plain1.c * plain2.b;
            det2 = plain1.c * plain2.a - plain1.a * plain2.c;
            det3 = plain1.a * plain2.b - plain1.b * plain2.a;
            param = Math.Sqrt(det1 * det1 + det2 * det2 + det3 * det3);
            this.alfa = det1 / param;
            this.beta = det2 / param;
            this.gama = det3 / param;
            Console.WriteLine(param);
        }

        public Line lineInPlain(Plain plain)
        {
            Line linereturned = new Line();
            Line norm = new Line(plain);
            linereturned.alfa = this.alfa - norm.alfa;
            linereturned.beta = this.beta - norm.beta;
            linereturned.gama = this.gama - norm.gama;
            return linereturned;
        }
        public double angleLinePlain(Plain plain)
        {
            double angle = 0;
            angle = Math.Asin(Math.Abs((this.alfa * plain.a + this.beta * plain.b + this.gama * plain.c) / Math.Sqrt(plain.a * plain.a + plain.b * plain.b + plain.c * plain.c))) * 180 / Math.PI;
            return angle;
        }
        public double angleLines(Line line)
        {
            double angle = 0;
            angle = Math.Acos(Math.Abs(this.alfa * line.alfa + this.beta * line.beta + this.gama * line.gama)) * 180 / Math.PI;
            return angle;
        }
    }
}

